"""
Python module generated from Java source file org.joml.Matrix2f

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import Externalizable
from java.io import IOException
from java.io import ObjectInput
from java.io import ObjectOutput
from java.text import DecimalFormat
from java.text import NumberFormat
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Matrix2f(Externalizable, Cloneable, Matrix2fc):
    """
    Contains the definition of a 2x2 matrix of floats, and associated functions to transform
    it. The matrix is column-major to match OpenGL's interpretation, and it looks like this:
    
         m00  m10
         m01  m11

    Author(s)
    - Joseph Burton
    """

    def __init__(self):
        """
        Create a new Matrix2f and set it to .identity() identity.
        """
        ...


    def __init__(self, mat: "Matrix2fc"):
        """
        Create a new Matrix2f and make it a copy of the given matrix.

        Arguments
        - mat: the Matrix2fc to copy the values from
        """
        ...


    def __init__(self, mat: "Matrix3fc"):
        """
        Create a new Matrix2f and make it a copy of the upper left 2x2 of the given Matrix3fc.

        Arguments
        - mat: the Matrix3fc to copy the values from
        """
        ...


    def __init__(self, m00: float, m01: float, m10: float, m11: float):
        """
        Create a new 2x2 matrix using the supplied float values. The order of the parameter is column-major,
        so the first two parameters specify the two elements of the first column.

        Arguments
        - m00: the value of m00
        - m01: the value of m01
        - m10: the value of m10
        - m11: the value of m11
        """
        ...


    def __init__(self, buffer: "FloatBuffer"):
        """
        Create a new Matrix2f by reading its 4 float components from the given FloatBuffer
        at the buffer's current position.
        
        That FloatBuffer is expected to hold the values in column-major order.
        
        The buffer's position will not be changed by this method.

        Arguments
        - buffer: the FloatBuffer to read the matrix values from
        """
        ...


    def __init__(self, col0: "Vector2fc", col1: "Vector2fc"):
        """
        Create a new Matrix2f and initialize its two columns using the supplied vectors.

        Arguments
        - col0: the first column
        - col1: the second column
        """
        ...


    def m00(self) -> float:
        ...


    def m01(self) -> float:
        ...


    def m10(self) -> float:
        ...


    def m11(self) -> float:
        ...


    def m00(self, m00: float) -> "Matrix2f":
        """
        Set the value of the matrix element at column 0 and row 0.

        Arguments
        - m00: the new value

        Returns
        - this
        """
        ...


    def m01(self, m01: float) -> "Matrix2f":
        """
        Set the value of the matrix element at column 0 and row 1.

        Arguments
        - m01: the new value

        Returns
        - this
        """
        ...


    def m10(self, m10: float) -> "Matrix2f":
        """
        Set the value of the matrix element at column 1 and row 0.

        Arguments
        - m10: the new value

        Returns
        - this
        """
        ...


    def m11(self, m11: float) -> "Matrix2f":
        """
        Set the value of the matrix element at column 1 and row 1.

        Arguments
        - m11: the new value

        Returns
        - this
        """
        ...


    def set(self, m: "Matrix2fc") -> "Matrix2f":
        """
        Set the elements of this matrix to the ones in `m`.

        Arguments
        - m: the matrix to copy the elements from

        Returns
        - this
        """
        ...


    def set(self, m: "Matrix3x2fc") -> "Matrix2f":
        """
        Set the elements of this matrix to the left 2x2 submatrix of `m`.

        Arguments
        - m: the matrix to copy the elements from

        Returns
        - this
        """
        ...


    def set(self, m: "Matrix3fc") -> "Matrix2f":
        """
        Set the elements of this matrix to the upper left 2x2 of the given Matrix3fc.

        Arguments
        - m: the Matrix3fc to copy the values from

        Returns
        - this
        """
        ...


    def mul(self, right: "Matrix2fc") -> "Matrix2f":
        """
        Multiply this matrix by the supplied `right` matrix.
        
        If `M` is `this` matrix and `R` the `right` matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        transformation of the right matrix will be applied first!

        Arguments
        - right: the right operand of the matrix multiplication

        Returns
        - this
        """
        ...


    def mul(self, right: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        ...


    def mulLocal(self, left: "Matrix2fc") -> "Matrix2f":
        """
        Pre-multiply this matrix by the supplied `left` matrix and store the result in `this`.
        
        If `M` is `this` matrix and `L` the `left` matrix,
        then the new matrix will be `L * M`. So when transforming a
        vector `v` with the new matrix by using `L * M * v`, the
        transformation of `this` matrix will be applied first!

        Arguments
        - left: the left operand of the matrix multiplication

        Returns
        - this
        """
        ...


    def mulLocal(self, left: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        ...


    def set(self, m00: float, m01: float, m10: float, m11: float) -> "Matrix2f":
        """
        Set the values within this matrix to the supplied float values. The result looks like this:
        
        m00, m10
        m01, m11

        Arguments
        - m00: the new value of m00
        - m01: the new value of m01
        - m10: the new value of m10
        - m11: the new value of m11

        Returns
        - this
        """
        ...


    def set(self, m: list[float]) -> "Matrix2f":
        """
        Set the values in this matrix based on the supplied double array. The result looks like this:
        
        0, 2
        1, 3
        
        This method only uses the first 4 values, all others are ignored.

        Arguments
        - m: the array to read the matrix values from

        Returns
        - this
        """
        ...


    def set(self, m: list[float], off: int) -> "Matrix2f":
        """
        Set the values in this matrix based on the supplied array in column-major order. The result looks like this:
        
        0, 2
        1, 3
        
        This method only uses the 4 values starting at the given offset.

        Arguments
        - m: the array to read the matrix values from
        - off: the offset into the array

        Returns
        - this
        """
        ...


    def set(self, col0: "Vector2fc", col1: "Vector2fc") -> "Matrix2f":
        """
        Set the two columns of this matrix to the supplied vectors, respectively.

        Arguments
        - col0: the first column
        - col1: the second column

        Returns
        - this
        """
        ...


    def determinant(self) -> float:
        ...


    def invert(self) -> "Matrix2f":
        """
        Invert this matrix.

        Returns
        - this
        """
        ...


    def invert(self, dest: "Matrix2f") -> "Matrix2f":
        ...


    def transpose(self) -> "Matrix2f":
        """
        Transpose this matrix.

        Returns
        - this
        """
        ...


    def transpose(self, dest: "Matrix2f") -> "Matrix2f":
        ...


    def toString(self) -> str:
        """
        Return a string representation of this matrix.
        
        This method creates a new DecimalFormat on every invocation with the format string "`0.000E0;-`".

        Returns
        - the string representation
        """
        ...


    def toString(self, formatter: "NumberFormat") -> str:
        """
        Return a string representation of this matrix by formatting the matrix elements with the given NumberFormat.

        Arguments
        - formatter: the NumberFormat used to format the matrix values with

        Returns
        - the string representation
        """
        ...


    def get(self, dest: "Matrix2f") -> "Matrix2f":
        """
        Get the current values of `this` matrix and store them into
        `dest`.
        
        This is the reverse method of .set(Matrix2fc) and allows to obtain
        intermediate calculation results when chaining multiple transformations.

        Arguments
        - dest: the destination matrix

        Returns
        - the passed in destination

        See
        - .set(Matrix2fc)
        """
        ...


    def get(self, dest: "Matrix3x2f") -> "Matrix3x2f":
        ...


    def get(self, dest: "Matrix3f") -> "Matrix3f":
        ...


    def getRotation(self) -> float:
        ...


    def get(self, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getTransposed(self, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def getTransposed(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def getTransposed(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getTransposed(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getToAddress(self, address: int) -> "Matrix2fc":
        ...


    def getTransposedToAddress(self, address: int) -> "Matrix2fc":
        ...


    def get(self, arr: list[float], offset: int) -> list[float]:
        ...


    def get(self, arr: list[float]) -> list[float]:
        ...


    def set(self, buffer: "FloatBuffer") -> "Matrix2f":
        """
        Set the values of this matrix by reading 4 float values from the given FloatBuffer in column-major order,
        starting at its current position.
        
        The FloatBuffer is expected to contain the values in column-major order.
        
        The position of the FloatBuffer will not be changed by this method.

        Arguments
        - buffer: the FloatBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def set(self, buffer: "ByteBuffer") -> "Matrix2f":
        """
        Set the values of this matrix by reading 4 float values from the given ByteBuffer in column-major order,
        starting at its current position.
        
        The ByteBuffer is expected to contain the values in column-major order.
        
        The position of the ByteBuffer will not be changed by this method.

        Arguments
        - buffer: the ByteBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def set(self, index: int, buffer: "FloatBuffer") -> "Matrix2f":
        """
        Set the values of this matrix by reading 4 float values from the given FloatBuffer in column-major order,
        starting at the specified absolute buffer position/index.
        
        The FloatBuffer is expected to contain the values in column-major order.
        
        The position of the FloatBuffer will not be changed by this method.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: the FloatBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def set(self, index: int, buffer: "ByteBuffer") -> "Matrix2f":
        """
        Set the values of this matrix by reading 4 float values from the given ByteBuffer in column-major order,
        starting at the specified absolute buffer position/index.
        
        The ByteBuffer is expected to contain the values in column-major order.
        
        The position of the ByteBuffer will not be changed by this method.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: the ByteBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def setFromAddress(self, address: int) -> "Matrix2f":
        """
        Set the values of this matrix by reading 4 float values from off-heap memory in column-major order,
        starting at the given address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap memory address to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def setTransposedFromAddress(self, address: int) -> "Matrix2f":
        """
        Set the values of this matrix by reading 4 float values from off-heap memory in row-major order,
        starting at the given address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap memory address to read the matrix values from in row-major order

        Returns
        - this
        """
        ...


    def zero(self) -> "Matrix2f":
        """
        Set all values within this matrix to zero.

        Returns
        - this
        """
        ...


    def identity(self) -> "Matrix2f":
        """
        Set this matrix to the identity.

        Returns
        - this
        """
        ...


    def scale(self, xy: "Vector2fc", dest: "Matrix2f") -> "Matrix2f":
        ...


    def scale(self, xy: "Vector2fc") -> "Matrix2f":
        """
        Apply scaling to this matrix by scaling the base axes by the given `xy.x` and
        `xy.y` factors, respectively.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the
        scaling will be applied first!

        Arguments
        - xy: the factors of the x and y component, respectively

        Returns
        - this
        """
        ...


    def scale(self, x: float, y: float, dest: "Matrix2f") -> "Matrix2f":
        ...


    def scale(self, x: float, y: float) -> "Matrix2f":
        """
        Apply scaling to this matrix by scaling the base axes by the given x and
        y factors.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`
        , the scaling will be applied first!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component

        Returns
        - this
        """
        ...


    def scale(self, xy: float, dest: "Matrix2f") -> "Matrix2f":
        ...


    def scale(self, xy: float) -> "Matrix2f":
        """
        Apply scaling to this matrix by uniformly scaling all base axes by the given `xy` factor.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`
        , the scaling will be applied first!

        Arguments
        - xy: the factor for all components

        Returns
        - this

        See
        - .scale(float, float)
        """
        ...


    def scaleLocal(self, x: float, y: float, dest: "Matrix2f") -> "Matrix2f":
        ...


    def scaleLocal(self, x: float, y: float) -> "Matrix2f":
        """
        Pre-multiply scaling to this matrix by scaling the base axes by the given x and
        y factors.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`, the
        scaling will be applied last!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component

        Returns
        - this
        """
        ...


    def scaling(self, factor: float) -> "Matrix2f":
        """
        Set this matrix to be a simple scale matrix, which scales all axes uniformly by the given factor.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional scaling.
        
        In order to post-multiply a scaling transformation directly to a
        matrix, use .scale(float) scale() instead.

        Arguments
        - factor: the scale factor in x and y

        Returns
        - this

        See
        - .scale(float)
        """
        ...


    def scaling(self, x: float, y: float) -> "Matrix2f":
        """
        Set this matrix to be a simple scale matrix.

        Arguments
        - x: the scale in x
        - y: the scale in y

        Returns
        - this
        """
        ...


    def scaling(self, xy: "Vector2fc") -> "Matrix2f":
        """
        Set this matrix to be a simple scale matrix which scales the base axes by `xy.x` and `xy.y` respectively.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional scaling.
        
        In order to post-multiply a scaling transformation directly to a
        matrix use .scale(Vector2fc) scale() instead.

        Arguments
        - xy: the scale in x and y respectively

        Returns
        - this

        See
        - .scale(Vector2fc)
        """
        ...


    def rotation(self, angle: float) -> "Matrix2f":
        """
        Set this matrix to a rotation matrix which rotates the given radians about the origin.
        
        The produced rotation will rotate a vector counter-clockwise around the origin.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional rotation.
        
        In order to post-multiply a rotation transformation directly to a
        matrix, use .rotate(float) rotate() instead.

        Arguments
        - angle: the angle in radians

        Returns
        - this

        See
        - .rotate(float)
        """
        ...


    def transform(self, v: "Vector2f") -> "Vector2f":
        ...


    def transform(self, v: "Vector2fc", dest: "Vector2f") -> "Vector2f":
        ...


    def transform(self, x: float, y: float, dest: "Vector2f") -> "Vector2f":
        ...


    def transformTranspose(self, v: "Vector2f") -> "Vector2f":
        ...


    def transformTranspose(self, v: "Vector2fc", dest: "Vector2f") -> "Vector2f":
        ...


    def transformTranspose(self, x: float, y: float, dest: "Vector2f") -> "Vector2f":
        ...


    def writeExternal(self, out: "ObjectOutput") -> None:
        ...


    def readExternal(self, in: "ObjectInput") -> None:
        ...


    def rotate(self, angle: float) -> "Matrix2f":
        """
        Apply rotation about the origin to this matrix by rotating the given amount of radians.
        
        The produced rotation will rotate a vector counter-clockwise around the origin.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`
        , the rotation will be applied first!
        
        Reference: <a href="https://en.wikipedia.org/wiki/Rotation_matrix#In_two_dimensions">http://en.wikipedia.org</a>

        Arguments
        - angle: the angle in radians

        Returns
        - this
        """
        ...


    def rotate(self, angle: float, dest: "Matrix2f") -> "Matrix2f":
        ...


    def rotateLocal(self, angle: float) -> "Matrix2f":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians about the origin.
        
        The produced rotation will rotate a vector counter-clockwise around the origin.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotation(float) rotation().
        
        Reference: <a href="https://en.wikipedia.org/wiki/Rotation_matrix#In_two_dimensions">http://en.wikipedia.org</a>

        Arguments
        - angle: the angle in radians to rotate about the X axis

        Returns
        - this

        See
        - .rotation(float)
        """
        ...


    def rotateLocal(self, angle: float, dest: "Matrix2f") -> "Matrix2f":
        ...


    def getRow(self, row: int, dest: "Vector2f") -> "Vector2f":
        ...


    def setRow(self, row: int, src: "Vector2fc") -> "Matrix2f":
        """
        Set the row at the given `row` index, starting with `0`.

        Arguments
        - row: the row index in `[0..1]`
        - src: the row components to set

        Returns
        - this

        Raises
        - IndexOutOfBoundsException: if `row` is not in `[0..1]`
        """
        ...


    def setRow(self, row: int, x: float, y: float) -> "Matrix2f":
        """
        Set the row at the given `row` index, starting with `0`.

        Arguments
        - row: the row index in `[0..1]`
        - x: the first element in the row
        - y: the second element in the row

        Returns
        - this

        Raises
        - IndexOutOfBoundsException: if `row` is not in `[0..1]`
        """
        ...


    def getColumn(self, column: int, dest: "Vector2f") -> "Vector2f":
        ...


    def setColumn(self, column: int, src: "Vector2fc") -> "Matrix2f":
        """
        Set the column at the given `column` index, starting with `0`.

        Arguments
        - column: the column index in `[0..1]`
        - src: the column components to set

        Returns
        - this

        Raises
        - IndexOutOfBoundsException: if `column` is not in `[0..1]`
        """
        ...


    def setColumn(self, column: int, x: float, y: float) -> "Matrix2f":
        """
        Set the column at the given `column` index, starting with `0`.

        Arguments
        - column: the column index in `[0..1]`
        - x: the first element in the column
        - y: the second element in the column

        Returns
        - this

        Raises
        - IndexOutOfBoundsException: if `column` is not in `[0..1]`
        """
        ...


    def get(self, column: int, row: int) -> float:
        ...


    def set(self, column: int, row: int, value: float) -> "Matrix2f":
        """
        Set the matrix element at the given column and row to the specified value.

        Arguments
        - column: the colum index in `[0..1]`
        - row: the row index in `[0..1]`
        - value: the value

        Returns
        - this
        """
        ...


    def normal(self) -> "Matrix2f":
        """
        Set `this` matrix to its own normal matrix.
        
        Please note that, if `this` is an orthogonal matrix or a matrix whose columns are orthogonal vectors,
        then this method *need not* be invoked, since in that case `this` itself is its normal matrix.
        In this case, use .set(Matrix2fc) to set a given Matrix2f to this matrix.

        Returns
        - this

        See
        - .set(Matrix2fc)
        """
        ...


    def normal(self, dest: "Matrix2f") -> "Matrix2f":
        """
        Compute a normal matrix from `this` matrix and store it into `dest`.
        
        Please note that, if `this` is an orthogonal matrix or a matrix whose columns are orthogonal vectors,
        then this method *need not* be invoked, since in that case `this` itself is its normal matrix.
        In this case, use .set(Matrix2fc) to set a given Matrix2f to this matrix.

        Arguments
        - dest: will hold the result

        Returns
        - dest

        See
        - .set(Matrix2fc)
        """
        ...


    def getScale(self, dest: "Vector2f") -> "Vector2f":
        ...


    def positiveX(self, dir: "Vector2f") -> "Vector2f":
        ...


    def normalizedPositiveX(self, dir: "Vector2f") -> "Vector2f":
        ...


    def positiveY(self, dir: "Vector2f") -> "Vector2f":
        ...


    def normalizedPositiveY(self, dir: "Vector2f") -> "Vector2f":
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def equals(self, m: "Matrix2fc", delta: float) -> bool:
        ...


    def swap(self, other: "Matrix2f") -> "Matrix2f":
        """
        Exchange the values of `this` matrix with the given `other` matrix.

        Arguments
        - other: the other matrix to exchange the values with

        Returns
        - this
        """
        ...


    def add(self, other: "Matrix2fc") -> "Matrix2f":
        """
        Component-wise add `this` and `other`.

        Arguments
        - other: the other addend

        Returns
        - this
        """
        ...


    def add(self, other: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        ...


    def sub(self, subtrahend: "Matrix2fc") -> "Matrix2f":
        """
        Component-wise subtract `subtrahend` from `this`.

        Arguments
        - subtrahend: the subtrahend

        Returns
        - this
        """
        ...


    def sub(self, other: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        ...


    def mulComponentWise(self, other: "Matrix2fc") -> "Matrix2f":
        """
        Component-wise multiply `this` by `other`.

        Arguments
        - other: the other matrix

        Returns
        - this
        """
        ...


    def mulComponentWise(self, other: "Matrix2fc", dest: "Matrix2f") -> "Matrix2f":
        ...


    def lerp(self, other: "Matrix2fc", t: float) -> "Matrix2f":
        """
        Linearly interpolate `this` and `other` using the given interpolation factor `t`
        and store the result in `this`.
        
        If `t` is `0.0` then the result is `this`. If the interpolation factor is `1.0`
        then the result is `other`.

        Arguments
        - other: the other matrix
        - t: the interpolation factor between 0.0 and 1.0

        Returns
        - this
        """
        ...


    def lerp(self, other: "Matrix2fc", t: float, dest: "Matrix2f") -> "Matrix2f":
        ...


    def isFinite(self) -> bool:
        ...


    def clone(self) -> "Object":
        ...



"""
Python module generated from Java source file org.bukkit.generator.WorldInfo

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import UUID
from org.bukkit import World
from org.bukkit.generator import *
from typing import Any, Callable, Iterable, Tuple


class WorldInfo:
    """
    Holds various information of a World
    """

    def getName(self) -> str:
        """
        Gets the unique name of this world

        Returns
        - Name of this world
        """
        ...


    def getUID(self) -> "UUID":
        """
        Gets the Unique ID of this world

        Returns
        - Unique ID of this world.
        """
        ...


    def getEnvironment(self) -> "World.Environment":
        """
        Gets the World.Environment type of this world

        Returns
        - This worlds Environment type
        """
        ...


    def getSeed(self) -> int:
        """
        Gets the Seed for this world.

        Returns
        - This worlds Seed
        """
        ...


    def getMinHeight(self) -> int:
        """
        Gets the minimum height of this world.
        
        If the min height is 0, there are only blocks from y=0.

        Returns
        - Minimum height of the world
        """
        ...


    def getMaxHeight(self) -> int:
        """
        Gets the maximum height of this world.
        
        If the max height is 100, there are only blocks from y=0 to y=99.

        Returns
        - Maximum height of the world
        """
        ...



"""
Python module generated from Java source file org.bukkit.UndefinedNullability

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class UndefinedNullability:
    """
    Annotation for types, whose nullability is not well defined, so
    org.jetbrains.annotations.NotNull nor
    org.jetbrains.annotations.Nullable is applicable. For example when
    interface defines a method, whose nullability depends on the implementation.

    Deprecated
    - This should generally not be used in any new API code as it
    suggests a bad API design.
    """

    def value(self) -> str:
        """
        Human readable description of the circumstances, in which the type is
        nullable.

        Returns
        - description
        """
        return ""



"""
Python module generated from Java source file com.google.gson.internal.JsonReaderInternalAccess

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.internal import *
from com.google.gson.stream import JsonReader
from java.io import IOException
from typing import Any, Callable, Iterable, Tuple


class JsonReaderInternalAccess:
    """
    Internal-only APIs of JsonReader available only to other classes in Gson.
    """

    INSTANCE = None


    def promoteNameToValue(self, reader: "JsonReader") -> None:
        """
        Changes the type of the current property name token to a string value.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.AllEqualOrdering

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from java.io import Serializable
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AllEqualOrdering(Ordering, Serializable):
    """
    An ordering that treats all references as equals, even nulls.

    Author(s)
    - Emily Soldal
    """

    def compare(self, left: "Object", right: "Object") -> int:
        ...


    def sortedCopy(self, iterable: Iterable["E"]) -> list["E"]:
        ...


    def immutableSortedCopy(self, iterable: Iterable["E"]) -> "ImmutableList"["E"]:
        ...


    def reverse(self) -> "Ordering"["S"]:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.nodes.CollectionNode

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.nodes import *
from typing import Any, Callable, Iterable, Tuple


class CollectionNode(Node):
    """
    Base class for the two collection types MappingNode mapping and SequenceNode
    collection.
    """

    def __init__(self, tag: "Tag", startMark: "Mark", endMark: "Mark", flowStyle: "DumperOptions.FlowStyle"):
        """
        Create

        Arguments
        - tag: - its tag
        - startMark: - start
        - endMark: - end
        - flowStyle: - style
        """
        ...


    def getValue(self) -> list["T"]:
        """
        Returns the elements in this sequence.

        Returns
        - Nodes in the specified order.
        """
        ...


    def getFlowStyle(self) -> "DumperOptions.FlowStyle":
        """
        Serialization style of this collection.

        Returns
        - `True` for flow style, `False` for block style.
        """
        ...


    def setFlowStyle(self, flowStyle: "DumperOptions.FlowStyle") -> None:
        """
        Setter

        Arguments
        - flowStyle: - flow style for collections
        """
        ...


    def setEndMark(self, endMark: "Mark") -> None:
        """
        Setter

        Arguments
        - endMark: - end
        """
        ...



"""
Python module generated from Java source file org.bukkit.conversations.ConversationFactory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.conversations import *
from org.bukkit.entity import Player
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class ConversationFactory:
    """
    A ConversationFactory is responsible for creating a Conversation
    from a predefined template. A ConversationFactory is typically created when
    a plugin is instantiated and builds a Conversation each time a user
    initiates a conversation with the plugin. Each Conversation maintains its
    own state and calls back as needed into the plugin.
    
    The ConversationFactory implements a fluid API, allowing parameters to be
    set as an extension to the constructor.
    """

    def __init__(self, plugin: "Plugin"):
        """
        Constructs a ConversationFactory.

        Arguments
        - plugin: The plugin that owns the factory.
        """
        ...


    def withModality(self, modal: bool) -> "ConversationFactory":
        """
        Sets the modality of all Conversations created by this factory.
        If a conversation is modal, all messages directed to the player are
        suppressed for the duration of the conversation.
        
        The default is True.

        Arguments
        - modal: The modality of all conversations to be created.

        Returns
        - This object.
        """
        ...


    def withLocalEcho(self, localEchoEnabled: bool) -> "ConversationFactory":
        """
        Sets the local echo status for all Conversations created by
        this factory. If local echo is enabled, any text submitted to a
        conversation gets echoed back into the submitter's chat window.

        Arguments
        - localEchoEnabled: The status of local echo.

        Returns
        - This object.
        """
        ...


    def withPrefix(self, prefix: "ConversationPrefix") -> "ConversationFactory":
        """
        Sets the ConversationPrefix that prepends all output from all
        generated conversations.
        
        The default is a NullConversationPrefix;

        Arguments
        - prefix: The ConversationPrefix to use.

        Returns
        - This object.
        """
        ...


    def withTimeout(self, timeoutSeconds: int) -> "ConversationFactory":
        """
        Sets the number of inactive seconds to wait before automatically
        abandoning all generated conversations.
        
        The default is 600 seconds (5 minutes).

        Arguments
        - timeoutSeconds: The number of seconds to wait.

        Returns
        - This object.
        """
        ...


    def withFirstPrompt(self, firstPrompt: "Prompt") -> "ConversationFactory":
        """
        Sets the first prompt to use in all generated conversations.
        
        The default is Prompt.END_OF_CONVERSATION.

        Arguments
        - firstPrompt: The first prompt.

        Returns
        - This object.
        """
        ...


    def withInitialSessionData(self, initialSessionData: dict["Object", "Object"]) -> "ConversationFactory":
        """
        Sets any initial data with which to populate the conversation context
        sessionData map.

        Arguments
        - initialSessionData: The conversation context's initial
            sessionData.

        Returns
        - This object.
        """
        ...


    def withEscapeSequence(self, escapeSequence: str) -> "ConversationFactory":
        """
        Sets the player input that, when received, will immediately terminate
        the conversation.

        Arguments
        - escapeSequence: Input to terminate the conversation.

        Returns
        - This object.
        """
        ...


    def withConversationCanceller(self, canceller: "ConversationCanceller") -> "ConversationFactory":
        """
        Adds a ConversationCanceller to constructed conversations.

        Arguments
        - canceller: The ConversationCanceller to add.

        Returns
        - This object.
        """
        ...


    def thatExcludesNonPlayersWithMessage(self, playerOnlyMessage: str) -> "ConversationFactory":
        """
        Prevents this factory from creating a conversation for non-player
        Conversable objects.

        Arguments
        - playerOnlyMessage: The message to return to a non-play in lieu of
            starting a conversation.

        Returns
        - This object.
        """
        ...


    def addConversationAbandonedListener(self, listener: "ConversationAbandonedListener") -> "ConversationFactory":
        """
        Adds a ConversationAbandonedListener to all conversations
        constructed by this factory.

        Arguments
        - listener: The listener to add.

        Returns
        - This object.
        """
        ...


    def buildConversation(self, forWhom: "Conversable") -> "Conversation":
        """
        Constructs a Conversation in accordance with the defaults set
        for this factory.

        Arguments
        - forWhom: The entity for whom the new conversation is mediating.

        Returns
        - A new conversation.
        """
        ...



"""
Python module generated from Java source file org.bukkit.material.Door

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit import TreeSpecies
from org.bukkit.block import BlockFace
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Door(MaterialData, Directional, Openable):
    """
    Represents a door.
    
    This class was previously deprecated, but has been retrofitted to
    work with modern doors. Some methods are undefined dependant on `isTopHalf()`
    due to Minecraft's internal representation of doors.

    See
    - Material.LEGACY_DARK_OAK_DOOR

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self):
        """
        Deprecated
        - Artifact of old API, equivalent to new `Door(Material.LEGACY_WOODEN_DOOR);`
        """
        ...


    def __init__(self, type: "Material"):
        ...


    def __init__(self, type: "Material", face: "BlockFace"):
        """
        Constructs the bottom half of a door of the given material type, facing the specified direction and set to closed

        Arguments
        - type: The type of material this door is made of. This must match the type of the block above.
        - face: The direction the door is facing.

        See
        - BlockFace.SOUTH
        """
        ...


    def __init__(self, type: "Material", face: "BlockFace", isOpen: bool):
        """
        Constructs the bottom half of a door of the given material type, facing the specified direction and set to open
        or closed

        Arguments
        - type: The type of material this door is made of. This must match the type of the block above.
        - face: The direction the door is facing.
        - isOpen: Whether the door is currently opened.

        See
        - BlockFace.SOUTH
        """
        ...


    def __init__(self, type: "Material", isHingeRight: bool):
        """
        Constructs the top half of door of the given material type and with the hinge on the left or right

        Arguments
        - type: The type of material this door is made of. This must match the type of the block below.
        - isHingeRight: True if the hinge is on the right hand side, False if the hinge is on the left hand side.

        See
        - Material.LEGACY_DARK_OAK_DOOR
        """
        ...


    def __init__(self, species: "TreeSpecies", face: "BlockFace"):
        """
        Constructs the bottom half of a wooden door of the given species, facing the specified direction and set to
        closed

        Arguments
        - species: The species this wooden door is made of. This must match the species of the block above.
        - face: The direction the door is facing.

        See
        - BlockFace.SOUTH
        """
        ...


    def __init__(self, species: "TreeSpecies", face: "BlockFace", isOpen: bool):
        """
        Constructs the bottom half of a wooden door of the given species, facing the specified direction and set to open
        or closed

        Arguments
        - species: The species this wooden door is made of. This must match the species of the block above.
        - face: The direction the door is facing.
        - isOpen: Whether the door is currently opened.

        See
        - BlockFace.SOUTH
        """
        ...


    def __init__(self, species: "TreeSpecies", isHingeRight: bool):
        """
        Constructs the top half of a wooden door of the given species and with the hinge on the left or right

        Arguments
        - species: The species this wooden door is made of. This must match the species of the block below.
        - isHingeRight: True if the hinge is on the right hand side, False if the hinge is on the left hand side.

        See
        - TreeSpecies
        """
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getWoodDoorOfSpecies(species: "TreeSpecies") -> "Material":
        """
        Returns the item type of a wooden door for the given tree species.

        Arguments
        - species: The species of wood door required.

        Returns
        - The item type for the given species.

        See
        - Material.LEGACY_DARK_OAK_DOOR
        """
        ...


    def isOpen(self) -> bool:
        """
        Result is undefined if `isTopHalf()` is True.
        """
        ...


    def setOpen(self, isOpen: bool) -> None:
        """
        Set whether the door is open. Undefined if `isTopHalf()` is True.
        """
        ...


    def isTopHalf(self) -> bool:
        """
        Returns
        - whether this is the top half of the door
        """
        ...


    def setTopHalf(self, isTopHalf: bool) -> None:
        """
        Configure this part of the door to be either the top or the bottom half

        Arguments
        - isTopHalf: True to make it the top half.
        """
        ...


    def getHingeCorner(self) -> "BlockFace":
        """
        Returns
        - BlockFace.SELF

        Deprecated
        - This method should not be used; use hinge and facing accessors instead.
        """
        ...


    def toString(self) -> str:
        ...


    def setFacingDirection(self, face: "BlockFace") -> None:
        """
        Set the direction that this door should is facing.
        
        Undefined if `isTopHalf()` is True.

        Arguments
        - face: the direction
        """
        ...


    def getFacing(self) -> "BlockFace":
        """
        Get the direction that this door is facing.
        
        Undefined if `isTopHalf()` is True.

        Returns
        - the direction
        """
        ...


    def getHinge(self) -> bool:
        """
        Returns the side of the door the hinge is on.
        
        Undefined if `isTopHalf()` is False.

        Returns
        - False for left hinge, True for right hinge
        """
        ...


    def setHinge(self, isHingeRight: bool) -> None:
        """
        Set whether the hinge is on the left or right side. Left is False, right is True.
        
        Undefined if `isTopHalf()` is False.

        Arguments
        - isHingeRight: True if the hinge is on the right hand side, False if the hinge is on the left hand side.
        """
        ...


    def clone(self) -> "Door":
        ...



"""
Python module generated from Java source file com.google.common.collect.TreeRangeMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import MoreObjects
from com.google.common.base import Predicate
from com.google.common.collect import *
from com.google.common.collect.Maps import IteratorBasedAbstractMap
from java.util import Collections
from java.util import Iterator
from java.util import NavigableMap
from java.util import NoSuchElementException
from java.util.function import BiFunction
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class TreeRangeMap(RangeMap):
    """
    An implementation of `RangeMap` based on a `TreeMap`, supporting all optional
    operations.
    
    Like all `RangeMap` implementations, this supports neither null keys nor null values.

    Author(s)
    - Louis Wasserman

    Since
    - 14.0
    """

    @staticmethod
    def create() -> "TreeRangeMap"["K", "V"]:
        ...


    def get(self, key: "K") -> "V":
        ...


    def getEntry(self, key: "K") -> "Entry"["Range"["K"], "V"]:
        ...


    def put(self, range: "Range"["K"], value: "V") -> None:
        ...


    def putCoalescing(self, range: "Range"["K"], value: "V") -> None:
        ...


    def putAll(self, rangeMap: "RangeMap"["K", "V"]) -> None:
        ...


    def clear(self) -> None:
        ...


    def span(self) -> "Range"["K"]:
        ...


    def remove(self, rangeToRemove: "Range"["K"]) -> None:
        ...


    def merge(self, range: "Range"["K"], value: "V", remappingFunction: "BiFunction"["V", "V", "V"]) -> None:
        """
        Since
        - 28.1
        """
        ...


    def asMapOfRanges(self) -> dict["Range"["K"], "V"]:
        ...


    def asDescendingMapOfRanges(self) -> dict["Range"["K"], "V"]:
        ...


    def subRangeMap(self, subRange: "Range"["K"]) -> "RangeMap"["K", "V"]:
        ...


    def equals(self, o: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file java.nio.charset.UnsupportedCharsetException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.charset import *
from typing import Any, Callable, Iterable, Tuple


class UnsupportedCharsetException(IllegalArgumentException):

    def __init__(self, charsetName: str):
        """
        Constructs an instance of this class.

        Arguments
        - charsetName: The name of the unsupported charset
        """
        ...


    def getCharsetName(self) -> str:
        """
        Retrieves the name of the unsupported charset.

        Returns
        - The name of the unsupported charset
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.world.ChunkEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Chunk
from org.bukkit.event.world import *
from typing import Any, Callable, Iterable, Tuple


class ChunkEvent(WorldEvent):
    """
    Represents a Chunk related event
    """

    def getChunk(self) -> "Chunk":
        """
        Gets the chunk being loaded/unloaded

        Returns
        - Chunk that triggered this event
        """
        ...



"""
Python module generated from Java source file org.joml.GeometryUtils

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class GeometryUtils:
    """
    Useful geometry methods.

    Author(s)
    - Richard Greenlees
    """

    @staticmethod
    def perpendicular(x: float, y: float, z: float, dest1: "Vector3f", dest2: "Vector3f") -> None:
        """
        Compute two arbitrary vectors perpendicular to the given normalized vector `(x, y, z)`, and store them in `dest1` and `dest2`,
        respectively.
        
        The computed vectors will themselves be perpendicular to each another and normalized. So the tree vectors `(x, y, z)`, `dest1` and
        `dest2` form an orthonormal basis.

        Arguments
        - x: the x coordinate of the normalized input vector
        - y: the y coordinate of the normalized input vector
        - z: the z coordinate of the normalized input vector
        - dest1: will hold the first perpendicular vector
        - dest2: will hold the second perpendicular vector
        """
        ...


    @staticmethod
    def perpendicular(v: "Vector3fc", dest1: "Vector3f", dest2: "Vector3f") -> None:
        """
        Compute two arbitrary vectors perpendicular to the given normalized vector `v`, and store them in `dest1` and `dest2`,
        respectively.
        
        The computed vectors will themselves be perpendicular to each another and normalized. So the tree vectors `v`, `dest1` and
        `dest2` form an orthonormal basis.

        Arguments
        - v: the Vector3f.normalize() normalized input vector
        - dest1: will hold the first perpendicular vector
        - dest2: will hold the second perpendicular vector
        """
        ...


    @staticmethod
    def normal(v0: "Vector3fc", v1: "Vector3fc", v2: "Vector3fc", dest: "Vector3f") -> None:
        """
        Calculate the normal of a surface defined by points `v1`, `v2` and `v3` and store it in `dest`.

        Arguments
        - v0: the first position
        - v1: the second position
        - v2: the third position
        - dest: will hold the result
        """
        ...


    @staticmethod
    def normal(v0X: float, v0Y: float, v0Z: float, v1X: float, v1Y: float, v1Z: float, v2X: float, v2Y: float, v2Z: float, dest: "Vector3f") -> None:
        """
        Calculate the normal of a surface defined by points `(v1X, v1Y, v1Z)`, `(v2X, v2Y, v2Z)` and `(v3X, v3Y, v3Z)`
        and store it in `dest`.

        Arguments
        - v0X: the x coordinate of the first position
        - v0Y: the y coordinate of the first position
        - v0Z: the z coordinate of the first position
        - v1X: the x coordinate of the second position
        - v1Y: the y coordinate of the second position
        - v1Z: the z coordinate of the second position
        - v2X: the x coordinate of the third position
        - v2Y: the y coordinate of the third position
        - v2Z: the z coordinate of the third position
        - dest: will hold the result
        """
        ...


    @staticmethod
    def tangent(v1: "Vector3fc", uv1: "Vector2fc", v2: "Vector3fc", uv2: "Vector2fc", v3: "Vector3fc", uv3: "Vector2fc", dest: "Vector3f") -> None:
        """
        Calculate the surface tangent for the three supplied vertices and UV coordinates and store the result in `dest`.

        Arguments
        - v1: XYZ of first vertex
        - uv1: UV of first vertex
        - v2: XYZ of second vertex
        - uv2: UV of second vertex
        - v3: XYZ of third vertex
        - uv3: UV of third vertex
        - dest: the tangent will be stored here
        """
        ...


    @staticmethod
    def bitangent(v1: "Vector3fc", uv1: "Vector2fc", v2: "Vector3fc", uv2: "Vector2fc", v3: "Vector3fc", uv3: "Vector2fc", dest: "Vector3f") -> None:
        """
        Calculate the surface bitangent for the three supplied vertices and UV coordinates and store the result in `dest`.

        Arguments
        - v1: XYZ of first vertex
        - uv1: UV of first vertex
        - v2: XYZ of second vertex
        - uv2: UV of second vertex
        - v3: XYZ of third vertex
        - uv3: UV of third vertex
        - dest: the binormal will be stored here
        """
        ...


    @staticmethod
    def tangentBitangent(v1: "Vector3fc", uv1: "Vector2fc", v2: "Vector3fc", uv2: "Vector2fc", v3: "Vector3fc", uv3: "Vector2fc", destTangent: "Vector3f", destBitangent: "Vector3f") -> None:
        """
        Calculate the surface tangent and bitangent for the three supplied vertices and UV coordinates and store the result in `dest`.

        Arguments
        - v1: XYZ of first vertex
        - uv1: UV of first vertex
        - v2: XYZ of second vertex
        - uv2: UV of second vertex
        - v3: XYZ of third vertex
        - uv3: UV of third vertex
        - destTangent: the tangent will be stored here
        - destBitangent: the bitangent will be stored here
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.tokens.ScalarToken

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.tokens import *
from typing import Any, Callable, Iterable, Tuple


class ScalarToken(Token):

    def __init__(self, value: str, startMark: "Mark", endMark: "Mark", plain: bool):
        ...


    def __init__(self, value: str, plain: bool, startMark: "Mark", endMark: "Mark", style: "DumperOptions.ScalarStyle"):
        ...


    def getPlain(self) -> bool:
        ...


    def getValue(self) -> str:
        ...


    def getStyle(self) -> "DumperOptions.ScalarStyle":
        ...


    def getTokenId(self) -> "Token.ID":
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.VillagerReplenishTradeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import AbstractVillager
from org.bukkit.entity import Villager
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from org.bukkit.inventory import MerchantRecipe
from typing import Any, Callable, Iterable, Tuple


class VillagerReplenishTradeEvent(EntityEvent, Cancellable):
    """
    Called when a Villager is about to restock one of its trades.
    
    If this event passes, the villager will reset the
    MerchantRecipe.getUses() uses of the affected .getRecipe()
    MerchantRecipe to `0`.
    """

    def __init__(self, what: "AbstractVillager", recipe: "MerchantRecipe"):
        ...


    def getRecipe(self) -> "MerchantRecipe":
        """
        Get the recipe to replenish.

        Returns
        - the replenished recipe
        """
        ...


    def setRecipe(self, recipe: "MerchantRecipe") -> None:
        """
        Set the recipe to replenish.

        Arguments
        - recipe: the replenished recipe
        """
        ...


    def getBonus(self) -> int:
        """
        Get the bonus uses added.

        Returns
        - the extra uses added

        Deprecated
        - MC 1.14 has changed how villagers restock their trades. Use
        MerchantRecipe.getUses().
        """
        ...


    def setBonus(self, bonus: int) -> None:
        """
        Set the bonus uses added.

        Arguments
        - bonus: the extra uses added

        Deprecated
        - MC 1.14 has changed how villagers restock their trades. This
        has no effect anymore.
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getEntity(self) -> "AbstractVillager":
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.util.BlockIterator

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import Iterator
from java.util import NoSuchElementException
from org.bukkit import Location
from org.bukkit import World
from org.bukkit.block import Block
from org.bukkit.block import BlockFace
from org.bukkit.entity import LivingEntity
from org.bukkit.util import *
from typing import Any, Callable, Iterable, Tuple


class BlockIterator(Iterator):
    """
    This class performs ray tracing and iterates along blocks on a line
    """

    def __init__(self, world: "World", start: "Vector", direction: "Vector", yOffset: float, maxDistance: int):
        """
        Constructs the BlockIterator.
        
        This considers all blocks as 1x1x1 in size.

        Arguments
        - world: The world to use for tracing
        - start: A Vector giving the initial location for the trace
        - direction: A Vector pointing in the direction for the trace
        - yOffset: The trace begins vertically offset from the start vector
            by this value
        - maxDistance: This is the maximum distance in blocks for the
            trace. Setting this value above 140 may lead to problems with
            unloaded chunks. A value of 0 indicates no limit
        """
        ...


    def __init__(self, loc: "Location", yOffset: float, maxDistance: int):
        """
        Constructs the BlockIterator.
        
        This considers all blocks as 1x1x1 in size.

        Arguments
        - loc: The location for the start of the ray trace
        - yOffset: The trace begins vertically offset from the start vector
            by this value
        - maxDistance: This is the maximum distance in blocks for the
            trace. Setting this value above 140 may lead to problems with
            unloaded chunks. A value of 0 indicates no limit
        """
        ...


    def __init__(self, loc: "Location", yOffset: float):
        ...


    def __init__(self, loc: "Location"):
        ...


    def __init__(self, entity: "LivingEntity", maxDistance: int):
        ...


    def __init__(self, entity: "LivingEntity"):
        ...


    def hasNext(self) -> bool:
        ...


    def next(self) -> "Block":
        """
        Returns the next Block in the trace

        Returns
        - the next Block in the trace
        """
        ...


    def remove(self) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.plugin.PluginLoader

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import File
from java.util.regex import Pattern
from org.bukkit.event import Event
from org.bukkit.event import Listener
from org.bukkit.plugin import *
from typing import Any, Callable, Iterable, Tuple


class PluginLoader:
    """
    Represents a plugin loader, which handles direct access to specific types
    of plugins
    """

    def loadPlugin(self, file: "File") -> "Plugin":
        """
        Loads the plugin contained in the specified file

        Arguments
        - file: File to attempt to load

        Returns
        - Plugin that was contained in the specified file, or null if
            unsuccessful

        Raises
        - InvalidPluginException: Thrown when the specified file is not a
            plugin
        - UnknownDependencyException: If a required dependency could not
            be found
        """
        ...


    def getPluginDescription(self, file: "File") -> "PluginDescriptionFile":
        """
        Loads a PluginDescriptionFile from the specified file

        Arguments
        - file: File to attempt to load from

        Returns
        - A new PluginDescriptionFile loaded from the plugin.yml in the
            specified file

        Raises
        - InvalidDescriptionException: If the plugin description file
            could not be created
        """
        ...


    def getPluginFileFilters(self) -> list["Pattern"]:
        """
        Returns a list of all filename filters expected by this PluginLoader

        Returns
        - The filters
        """
        ...


    def createRegisteredListeners(self, listener: "Listener", plugin: "Plugin") -> dict[type["Event"], set["RegisteredListener"]]:
        """
        Creates and returns registered listeners for the event classes used in
        this listener

        Arguments
        - listener: The object that will handle the eventual call back
        - plugin: The plugin to use when creating registered listeners

        Returns
        - The registered listeners.
        """
        ...


    def enablePlugin(self, plugin: "Plugin") -> None:
        """
        Enables the specified plugin
        
        Attempting to enable a plugin that is already enabled will have no
        effect

        Arguments
        - plugin: Plugin to enable
        """
        ...


    def disablePlugin(self, plugin: "Plugin") -> None:
        """
        Disables the specified plugin
        
        Attempting to disable a plugin that is not enabled will have no effect

        Arguments
        - plugin: Plugin to disable
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.Platform

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from java.util import Arrays
from java.util.concurrent import ConcurrentHashMap
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Platform:
    """
    Methods factored out so that they can be emulated differently in GWT.

    Author(s)
    - Hayward Chan
    """





"""
Python module generated from Java source file com.google.common.collect.ForwardingConcurrentMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util.concurrent import ConcurrentMap
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class ForwardingConcurrentMap(ForwardingMap, ConcurrentMap):
    """
    A concurrent map which forwards all its method calls to another concurrent map. Subclasses should
    override one or more methods to modify the behavior of the backing map as desired per the <a
    href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.
    
    **`default` method warning:** This class forwards calls to *only some* `default` methods. Specifically, it forwards calls only for methods that existed <a
    href="https://docs.oracle.com/javase/7/docs/api/java/util/concurrent/ConcurrentMap.html">before
    `default` methods were introduced</a>. For newer methods, like `forEach`, it inherits
    their default implementations. When those implementations invoke methods, they invoke methods on
    the `ForwardingConcurrentMap`.

    Author(s)
    - Charles Fry

    Since
    - 2.0
    """

    def putIfAbsent(self, key: "K", value: "V") -> "V":
        ...


    def remove(self, key: "Object", value: "Object") -> bool:
        ...


    def replace(self, key: "K", value: "V") -> "V":
        ...


    def replace(self, key: "K", oldValue: "V", newValue: "V") -> bool:
        ...



"""
Python module generated from Java source file org.bukkit.inventory.ComplexRecipe

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Keyed
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class ComplexRecipe(Recipe, Keyed):
    """
    Represents a complex recipe which has imperative server-defined behavior, eg
    armor dyeing.
    
    Note: Since a complex recipe has dynamic outputs, .getResult() will
    sometimes return an AIR ItemStack.
    """





"""
Python module generated from Java source file java.io.BufferedReader

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from java.util import Iterator
from java.util import NoSuchElementException
from java.util import Objects
from java.util import Spliterator
from java.util.stream import Stream
from java.util.stream import StreamSupport
from typing import Any, Callable, Iterable, Tuple


class BufferedReader(Reader):

    def __init__(self, in: "Reader", sz: int):
        """
        Creates a buffering character-input stream that uses an input buffer of
        the specified size.

        Arguments
        - in: A Reader
        - sz: Input-buffer size

        Raises
        - IllegalArgumentException: If `sz <= 0`
        """
        ...


    def __init__(self, in: "Reader"):
        """
        Creates a buffering character-input stream that uses a default-sized
        input buffer.

        Arguments
        - in: A Reader
        """
        ...


    def read(self) -> int:
        """
        Reads a single character.

        Returns
        - The character read, as an integer in the range
                0 to 65535 (`0x00-0xffff`), or -1 if the
                end of the stream has been reached

        Raises
        - IOException: If an I/O error occurs
        """
        ...


    def read(self, cbuf: list[str], off: int, len: int) -> int:
        """
        Reads characters into a portion of an array.
        
         This method implements the general contract of the corresponding
        Reader.read(char[], int, int) read method of the
        Reader class.  As an additional convenience, it
        attempts to read as many characters as possible by repeatedly invoking
        the `read` method of the underlying stream.  This iterated
        `read` continues until one of the following conditions becomes
        True:
        
        
          -  The specified number of characters have been read,
        
          -  The `read` method of the underlying stream returns
          `-1`, indicating end-of-file, or
        
          -  The `ready` method of the underlying stream
          returns `False`, indicating that further input requests
          would block.
        
        
        If the first `read` on the underlying stream returns
        `-1` to indicate end-of-file then this method returns
        `-1`.  Otherwise this method returns the number of characters
        actually read.
        
         Subclasses of this class are encouraged, but not required, to
        attempt to read as many characters as possible in the same fashion.
        
         Ordinarily this method takes characters from this stream's character
        buffer, filling it from the underlying stream as necessary.  If,
        however, the buffer is empty, the mark is not valid, and the requested
        length is at least as large as the buffer, then this method will read
        characters directly from the underlying stream into the given array.
        Thus redundant `BufferedReader`s will not copy data
        unnecessarily.

        Arguments
        - cbuf: 
        - off: 
        - len: 

        Returns
        - 

        Raises
        - IndexOutOfBoundsException: 
        - IOException: 
        """
        ...


    def readLine(self) -> str:
        """
        Reads a line of text.  A line is considered to be terminated by any one
        of a line feed ('\n'), a carriage return ('\r'), a carriage return
        followed immediately by a line feed, or by reaching the end-of-file
        (EOF).

        Returns
        - A String containing the contents of the line, not including
                    any line-termination characters, or null if the end of the
                    stream has been reached without reading any characters

        Raises
        - IOException: If an I/O error occurs

        See
        - java.nio.file.Files.readAllLines
        """
        ...


    def skip(self, n: int) -> int:
        """

        """
        ...


    def ready(self) -> bool:
        """
        Tells whether this stream is ready to be read.  A buffered character
        stream is ready if the buffer is not empty, or if the underlying
        character stream is ready.

        Raises
        - IOException: If an I/O error occurs
        """
        ...


    def markSupported(self) -> bool:
        """
        Tells whether this stream supports the mark() operation, which it does.
        """
        ...


    def mark(self, readAheadLimit: int) -> None:
        """
        Marks the present position in the stream.  Subsequent calls to reset()
        will attempt to reposition the stream to this point.

        Arguments
        - readAheadLimit: Limit on the number of characters that may be
                                read while still preserving the mark. An attempt
                                to reset the stream after reading characters
                                up to this limit or beyond may fail.
                                A limit value larger than the size of the input
                                buffer will cause a new buffer to be allocated
                                whose size is no smaller than limit.
                                Therefore large values should be used with care.

        Raises
        - IllegalArgumentException: If `readAheadLimit < 0`
        - IOException: If an I/O error occurs
        """
        ...


    def reset(self) -> None:
        """
        Resets the stream to the most recent mark.

        Raises
        - IOException: If the stream has never been marked,
                                 or if the mark has been invalidated
        """
        ...


    def close(self) -> None:
        ...


    def lines(self) -> "Stream"[str]:
        """
        Returns a `Stream`, the elements of which are lines read from
        this `BufferedReader`.  The Stream is lazily populated,
        i.e., read only occurs during the
        <a href="../util/stream/package-summary.html#StreamOps">terminal
        stream operation</a>.
        
         The reader must not be operated on during the execution of the
        terminal stream operation. Otherwise, the result of the terminal stream
        operation is undefined.
        
         After execution of the terminal stream operation there are no
        guarantees that the reader will be at a specific position from which to
        read the next character or line.
        
         If an IOException is thrown when accessing the underlying
        `BufferedReader`, it is wrapped in an UncheckedIOException which will be thrown from the `Stream`
        method that caused the read to take place. This method will return a
        Stream if invoked on a BufferedReader that is closed. Any operation on
        that stream that requires reading from the BufferedReader after it is
        closed, will cause an UncheckedIOException to be thrown.

        Returns
        - a `Stream<String>` providing the lines of text
                described by this `BufferedReader`

        Since
        - 1.8
        """
        ...



"""
Python module generated from Java source file org.bukkit.util.BlockVector

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.configuration.serialization import SerializableAs
from org.bukkit.util import *
from typing import Any, Callable, Iterable, Tuple


class BlockVector(Vector):
    """
    A vector with a hash function that floors the X, Y, Z components, a la
    BlockVector in WorldEdit. BlockVectors can be used in hash sets and
    hash maps. Be aware that BlockVectors are mutable, but it is important
    that BlockVectors are never changed once put into a hash set or hash map.
    """

    def __init__(self):
        """
        Construct the vector with all components as 0.
        """
        ...


    def __init__(self, vec: "Vector"):
        """
        Construct the vector with another vector.

        Arguments
        - vec: The other vector.
        """
        ...


    def __init__(self, x: int, y: int, z: int):
        """
        Construct the vector with provided integer components.

        Arguments
        - x: X component
        - y: Y component
        - z: Z component
        """
        ...


    def __init__(self, x: float, y: float, z: float):
        """
        Construct the vector with provided double components.

        Arguments
        - x: X component
        - y: Y component
        - z: Z component
        """
        ...


    def __init__(self, x: float, y: float, z: float):
        """
        Construct the vector with provided float components.

        Arguments
        - x: X component
        - y: Y component
        - z: Z component
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Checks if another object is equivalent.

        Arguments
        - obj: The other object

        Returns
        - whether the other object is equivalent
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hash code for this vector.

        Returns
        - hash code
        """
        ...


    def clone(self) -> "BlockVector":
        """
        Get a new block vector.

        Returns
        - vector
        """
        ...


    @staticmethod
    def deserialize(args: dict[str, "Object"]) -> "BlockVector":
        ...



"""
Python module generated from Java source file java.net.UnknownHostException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.net import *
from typing import Any, Callable, Iterable, Tuple


class UnknownHostException(IOException):
    """
    Thrown to indicate that the IP address of a host could not be determined.

    Author(s)
    - Jonathan Payne

    Since
    - 1.0
    """

    def __init__(self, message: str):
        """
        Constructs a new `UnknownHostException` with the
        specified detail message.

        Arguments
        - message: the detail message.
        """
        ...


    def __init__(self):
        """
        Constructs a new `UnknownHostException` with no detail
        message.
        """
        ...



"""
Python module generated from Java source file com.google.common.io.BaseEncoding

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Ascii
from com.google.common.io import *
from com.google.errorprone.annotations.concurrent import LazyInit
from java.io import IOException
from java.io import InputStream
from java.io import OutputStream
from java.io import Reader
from java.io import Writer
from java.util import Arrays
from java.util import Objects
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class BaseEncoding:
    """
    A binary encoding scheme for reversibly translating between byte sequences and printable ASCII
    strings. This class includes several constants for encoding schemes specified by <a
    href="http://tools.ietf.org/html/rfc4648">RFC 4648</a>. For example, the expression:
    
    ````BaseEncoding.base32().encode("foo".getBytes(US_ASCII))````
    
    returns the string `"MZXW6==="`, and
    
    ````byte[] decoded = BaseEncoding.base32().decode("MZXW6===");````
    
    ...returns the ASCII bytes of the string `"foo"`.
    
    By default, `BaseEncoding`'s behavior is relatively strict and in accordance with RFC
    4648. Decoding rejects characters in the wrong case, though padding is optional. To modify
    encoding and decoding behavior, use configuration methods to obtain a new encoding with modified
    behavior:
    
    ````BaseEncoding.base16().lowerCase().decode("deadbeef");````
    
    Warning: BaseEncoding instances are immutable. Invoking a configuration method has no effect
    on the receiving instance; you must store and use the new encoding instance it returns, instead.
    
    ````// Do NOT do this
    BaseEncoding hex = BaseEncoding.base16();
    hex.lowerCase(); // does nothing!
    return hex.decode("deadbeef"); // throws an IllegalArgumentException````
    
    It is guaranteed that `encoding.decode(encoding.encode(x))` is always equal to `x`, but the reverse does not necessarily hold.
    
    <table>
    <caption>Encodings</caption>
    <tr>
    <th>Encoding
    <th>Alphabet
    <th>`char:byte` ratio
    <th>Default padding
    <th>Comments
    <tr>
    <td>.base16()
    <td>0-9 A-F
    <td>2.00
    <td>N/A
    <td>Traditional hexadecimal. Defaults to upper case.
    <tr>
    <td>.base32()
    <td>A-Z 2-7
    <td>1.60
    <td>=
    <td>Human-readable; no possibility of mixing up 0/O or 1/I. Defaults to upper case.
    <tr>
    <td>.base32Hex()
    <td>0-9 A-V
    <td>1.60
    <td>=
    <td>"Numerical" base 32; extended from the traditional hex alphabet. Defaults to upper case.
    <tr>
    <td>.base64()
    <td>A-Z a-z 0-9 + /
    <td>1.33
    <td>=
    <td>
    <tr>
    <td>.base64Url()
    <td>A-Z a-z 0-9 - _
    <td>1.33
    <td>=
    <td>Safe to use as filenames, or to pass in URLs without escaping
    </table>
    
    All instances of this class are immutable, so they may be stored safely as static constants.

    Author(s)
    - Louis Wasserman

    Since
    - 14.0
    """

    def encode(self, bytes: list[int]) -> str:
        """
        Encodes the specified byte array, and returns the encoded `String`.
        """
        ...


    def encode(self, bytes: list[int], off: int, len: int) -> str:
        """
        Encodes the specified range of the specified byte array, and returns the encoded `String`.
        """
        ...


    def encodingStream(self, writer: "Writer") -> "OutputStream":
        """
        Returns an `OutputStream` that encodes bytes using this encoding into the specified
        `Writer`. When the returned `OutputStream` is closed, so is the backing `Writer`.
        """
        ...


    def encodingSink(self, encodedSink: "CharSink") -> "ByteSink":
        """
        Returns a `ByteSink` that writes base-encoded bytes to the specified `CharSink`.
        """
        ...


    def canDecode(self, chars: "CharSequence") -> bool:
        """
        Determines whether the specified character sequence is a valid encoded string according to this
        encoding.

        Since
        - 20.0
        """
        ...


    def decode(self, chars: "CharSequence") -> list[int]:
        """
        Decodes the specified character sequence, and returns the resulting `byte[]`. This is the
        inverse operation to .encode(byte[]).

        Raises
        - IllegalArgumentException: if the input is not a valid encoded string according to this
            encoding.
        """
        ...


    def decodingStream(self, reader: "Reader") -> "InputStream":
        """
        Returns an `InputStream` that decodes base-encoded input from the specified `Reader`. The returned stream throws a DecodingException upon decoding-specific errors.
        """
        ...


    def decodingSource(self, encodedSource: "CharSource") -> "ByteSource":
        """
        Returns a `ByteSource` that reads base-encoded bytes from the specified `CharSource`.
        """
        ...


    def omitPadding(self) -> "BaseEncoding":
        """
        Returns an encoding that behaves equivalently to this encoding, but omits any padding
        characters as specified by <a href="http://tools.ietf.org/html/rfc4648#section-3.2">RFC 4648
        section 3.2</a>, Padding of Encoded Data.
        """
        ...


    def withPadChar(self, padChar: str) -> "BaseEncoding":
        """
        Returns an encoding that behaves equivalently to this encoding, but uses an alternate character
        for padding.

        Raises
        - IllegalArgumentException: if this padding character is already used in the alphabet or a
            separator
        """
        ...


    def withSeparator(self, separator: str, n: int) -> "BaseEncoding":
        """
        Returns an encoding that behaves equivalently to this encoding, but adds a separator string
        after every `n` characters. Any occurrences of any characters that occur in the separator
        are skipped over in decoding.

        Raises
        - IllegalArgumentException: if any alphabet or padding characters appear in the separator
            string, or if `n <= 0`
        - UnsupportedOperationException: if this encoding already uses a separator
        """
        ...


    def upperCase(self) -> "BaseEncoding":
        """
        Returns an encoding that behaves equivalently to this encoding, but encodes and decodes with
        uppercase letters. Padding and separator characters remain in their original case.

        Raises
        - IllegalStateException: if the alphabet used by this encoding contains mixed upper- and
            lower-case characters
        """
        ...


    def lowerCase(self) -> "BaseEncoding":
        """
        Returns an encoding that behaves equivalently to this encoding, but encodes and decodes with
        lowercase letters. Padding and separator characters remain in their original case.

        Raises
        - IllegalStateException: if the alphabet used by this encoding contains mixed upper- and
            lower-case characters
        """
        ...


    def ignoreCase(self) -> "BaseEncoding":
        """
        Returns an encoding that behaves equivalently to this encoding, but decodes letters without
        regard to case.

        Raises
        - IllegalStateException: if the alphabet used by this encoding contains mixed upper- and
            lower-case characters

        Since
        - 32.0.0
        """
        ...


    @staticmethod
    def base64() -> "BaseEncoding":
        """
        The "base64" base encoding specified by <a
        href="http://tools.ietf.org/html/rfc4648#section-4">RFC 4648 section 4</a>, Base 64 Encoding.
        (This is the same as the base 64 encoding from <a
        href="http://tools.ietf.org/html/rfc3548#section-3">RFC 3548</a>.)
        
        The character `'='` is used for padding, but can be .omitPadding()
        omitted or .withPadChar(char) replaced.
        
        No line feeds are added by default, as per <a
        href="http://tools.ietf.org/html/rfc4648#section-3.1">RFC 4648 section 3.1</a>, Line Feeds in
        Encoded Data. Line feeds may be added using .withSeparator(String, int).
        """
        ...


    @staticmethod
    def base64Url() -> "BaseEncoding":
        """
        The "base64url" encoding specified by <a
        href="http://tools.ietf.org/html/rfc4648#section-5">RFC 4648 section 5</a>, Base 64 Encoding
        with URL and Filename Safe Alphabet, also sometimes referred to as the "web safe Base64." (This
        is the same as the base 64 encoding with URL and filename safe alphabet from <a
        href="http://tools.ietf.org/html/rfc3548#section-4">RFC 3548</a>.)
        
        The character `'='` is used for padding, but can be .omitPadding()
        omitted or .withPadChar(char) replaced.
        
        No line feeds are added by default, as per <a
        href="http://tools.ietf.org/html/rfc4648#section-3.1">RFC 4648 section 3.1</a>, Line Feeds in
        Encoded Data. Line feeds may be added using .withSeparator(String, int).
        """
        ...


    @staticmethod
    def base32() -> "BaseEncoding":
        """
        The "base32" encoding specified by <a href="http://tools.ietf.org/html/rfc4648#section-6">RFC
        4648 section 6</a>, Base 32 Encoding. (This is the same as the base 32 encoding from <a
        href="http://tools.ietf.org/html/rfc3548#section-5">RFC 3548</a>.)
        
        The character `'='` is used for padding, but can be .omitPadding()
        omitted or .withPadChar(char) replaced.
        
        No line feeds are added by default, as per <a
        href="http://tools.ietf.org/html/rfc4648#section-3.1">RFC 4648 section 3.1</a>, Line Feeds in
        Encoded Data. Line feeds may be added using .withSeparator(String, int).
        """
        ...


    @staticmethod
    def base32Hex() -> "BaseEncoding":
        """
        The "base32hex" encoding specified by <a
        href="http://tools.ietf.org/html/rfc4648#section-7">RFC 4648 section 7</a>, Base 32 Encoding
        with Extended Hex Alphabet. There is no corresponding encoding in RFC 3548.
        
        The character `'='` is used for padding, but can be .omitPadding()
        omitted or .withPadChar(char) replaced.
        
        No line feeds are added by default, as per <a
        href="http://tools.ietf.org/html/rfc4648#section-3.1">RFC 4648 section 3.1</a>, Line Feeds in
        Encoded Data. Line feeds may be added using .withSeparator(String, int).
        """
        ...


    @staticmethod
    def base16() -> "BaseEncoding":
        """
        The "base16" encoding specified by <a href="http://tools.ietf.org/html/rfc4648#section-8">RFC
        4648 section 8</a>, Base 16 Encoding. (This is the same as the base 16 encoding from <a
        href="http://tools.ietf.org/html/rfc3548#section-6">RFC 3548</a>.) This is commonly known as
        "hexadecimal" format.
        
        No padding is necessary in base 16, so .withPadChar(char) and .omitPadding()
        have no effect.
        
        No line feeds are added by default, as per <a
        href="http://tools.ietf.org/html/rfc4648#section-3.1">RFC 4648 section 3.1</a>, Line Feeds in
        Encoded Data. Line feeds may be added using .withSeparator(String, int).
        """
        ...


    class DecodingException(IOException):
        """
        Exception indicating invalid base-encoded input encountered while decoding.

    Author(s)
        - Louis Wasserman

        Since
        - 15.0
        """





"""
Python module generated from Java source file org.bukkit.command.CommandException

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.command import *
from typing import Any, Callable, Iterable, Tuple


class CommandException(RuntimeException):
    """
    Thrown when an unhandled exception occurs during the execution of a Command
    """

    def __init__(self):
        """
        Creates a new instance of `CommandException` without detail
        message.
        """
        ...


    def __init__(self, msg: str):
        """
        Constructs an instance of `CommandException` with the
        specified detail message.

        Arguments
        - msg: the detail message.
        """
        ...


    def __init__(self, msg: str, cause: "Throwable"):
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.nodes.NodeId

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.yaml.snakeyaml.nodes import *
from typing import Any, Callable, Iterable, Tuple


class NodeId(Enum):
    """
    Enum for the basic YAML types: scalar, sequence, mapping or anchor.
    """

    scalar = 0
    sequence = 1
    mapping = 2
    anchor = 3



"""
Python module generated from Java source file org.bukkit.event.inventory.TradeSelectEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.event import HandlerList
from org.bukkit.event.inventory import *
from org.bukkit.inventory import Merchant
from org.bukkit.inventory import MerchantInventory
from org.bukkit.inventory.view import MerchantView
from typing import Any, Callable, Iterable, Tuple


class TradeSelectEvent(InventoryInteractEvent):
    """
    This event is called whenever a player clicks a new trade on the trades
    sidebar.
    
    This event allows the user to get the index of the trade, letting them get
    the MerchantRecipe via the Merchant.
    """

    def __init__(self, transaction: "MerchantView", newIndex: int):
        ...


    def getIndex(self) -> int:
        """
        Used to get the index of the trade the player clicked on.

        Returns
        - The index of the trade clicked by the player
        """
        ...


    def getInventory(self) -> "MerchantInventory":
        ...


    def getMerchant(self) -> "Merchant":
        """
        Get the Merchant involved.

        Returns
        - the Merchant
        """
        ...


    def getView(self) -> "MerchantView":
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.SequentialExecutor

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Preconditions
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations.concurrent import GuardedBy
from com.google.errorprone.annotations.concurrent import LazyInit
from com.google.j2objc.annotations import RetainedWith
from java.util import ArrayDeque
from java.util import Deque
from java.util.concurrent import Executor
from java.util.concurrent import RejectedExecutionException
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class SequentialExecutor(Executor):
    """
    Executor ensuring that all Runnables submitted are executed in order, using the provided
    Executor, and sequentially such that no two will ever be running at the same time.
    
    Tasks submitted to .execute(Runnable) are executed in FIFO order.
    
    The execution of tasks is done by one thread as long as there are tasks left in the queue.
    When a task is Thread.interrupt interrupted, execution of subsequent tasks
    continues. See QueueWorker.workOnQueue for details.
    
    `RuntimeException`s thrown by tasks are simply logged and the executor keeps trucking.
    If an `Error` is thrown, the error will propagate and execution will stop until it is
    restarted by a call to .execute.
    """

    def execute(self, task: "Runnable") -> None:
        """
        Adds a task to the queue and makes sure a worker thread is running.
        
        If this method throws, e.g. a `RejectedExecutionException` from the delegate executor,
        execution of tasks will stop until a call to this method is made.
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.entity.Snowman

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Snowman(Golem):
    """
    Represents a snowman entity
    """

    def isDerp(self) -> bool:
        """
        Gets whether this snowman is in "derp mode", meaning it is not wearing a
        pumpkin.

        Returns
        - True if the snowman is bald, False if it is wearing a pumpkin
        """
        ...


    def setDerp(self, derpMode: bool) -> None:
        """
        Sets whether this snowman is in "derp mode", meaning it is not wearing a
        pumpkin. NOTE: This value is not persisted to disk and will therefore
        reset when the chunk is reloaded.

        Arguments
        - derpMode: True to remove the pumpkin, False to add a pumpkin
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockReceiveGameEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import GameEvent
from org.bukkit.block import Block
from org.bukkit.entity import Entity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class BlockReceiveGameEvent(BlockEvent, Cancellable):
    """
    Called when a Sculk sensor receives a game event and hence might activate.
    
    Will be called cancelled if the block's default behavior is to ignore the
    event.
    """

    def __init__(self, event: "GameEvent", block: "Block", entity: "Entity"):
        ...


    def getEvent(self) -> "GameEvent":
        """
        Get the underlying event.

        Returns
        - the event
        """
        ...


    def getEntity(self) -> "Entity":
        """
        Get the entity which triggered this event, if present.

        Returns
        - triggering entity or null
        """
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def isCancelled(self) -> bool:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file java.util.concurrent.CancellationException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class CancellationException(IllegalStateException):
    """
    Exception indicating that the result of a value-producing task,
    such as a FutureTask, cannot be retrieved because the task
    was cancelled.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self):
        """
        Constructs a `CancellationException` with no detail message.
        """
        ...


    def __init__(self, message: str):
        """
        Constructs a `CancellationException` with the specified detail
        message.

        Arguments
        - message: the detail message
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.SuspiciousStewMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory.meta import *
from org.bukkit.potion import PotionEffect
from org.bukkit.potion import PotionEffectType
from typing import Any, Callable, Iterable, Tuple


class SuspiciousStewMeta(ItemMeta):
    """
    Represents a suspicious stew that can have custom effects.
    """

    def hasCustomEffects(self) -> bool:
        """
        Checks for the presence of custom potion effects.

        Returns
        - True if custom potion effects are applied
        """
        ...


    def getCustomEffects(self) -> list["PotionEffect"]:
        """
        Gets an immutable list containing all custom potion effects applied to
        this suspicious stew.
        
        Plugins should check that hasCustomEffects() returns True before calling
        this method.

        Returns
        - the immutable list of custom potion effects
        """
        ...


    def addCustomEffect(self, effect: "PotionEffect", overwrite: bool) -> bool:
        """
        Adds a custom potion effect to this suspicious stew.

        Arguments
        - effect: the potion effect to add
        - overwrite: True if any existing effect of the same type should be
        overwritten

        Returns
        - True if the suspicious stew meta changed as a result of this call
        """
        ...


    def removeCustomEffect(self, type: "PotionEffectType") -> bool:
        """
        Removes a custom potion effect from this suspicious stew.

        Arguments
        - type: the potion effect type to remove

        Returns
        - True if the suspicious stew meta changed as a result of this call
        """
        ...


    def hasCustomEffect(self, type: "PotionEffectType") -> bool:
        """
        Checks for a specific custom potion effect type on this suspicious stew.

        Arguments
        - type: the potion effect type to check for

        Returns
        - True if the suspicious stew has this effect
        """
        ...


    def clearCustomEffects(self) -> bool:
        """
        Removes all custom potion effects from this suspicious stew.

        Returns
        - True if the suspicious stew meta changed as a result of this call
        """
        ...


    def clone(self) -> "SuspiciousStewMeta":
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.components.ToolComponent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit import Tag
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.inventory.meta.components import *
from typing import Any, Callable, Iterable, Tuple


class ToolComponent(ConfigurationSerializable):
    """
    Represents a component which can turn any item into a tool.
    """

    def getDefaultMiningSpeed(self) -> float:
        """
        Get the default mining speed of this tool. This value is used by the tool
        if no rule explicitly overrides it. 1.0 is standard mining speed.

        Returns
        - the default mining speed

        See
        - ToolRule.getSpeed()
        """
        ...


    def setDefaultMiningSpeed(self, speed: float) -> None:
        """
        Set the default mining speed of this tool. This value is used by the tool
        if no rule explicitly overrides it. 1.0 is standard mining speed.

        Arguments
        - speed: the speed to set
        """
        ...


    def getDamagePerBlock(self) -> int:
        """
        Get the amount of durability to be removed from the tool each time a
        block is broken.

        Returns
        - the damage per block
        """
        ...


    def setDamagePerBlock(self, damage: int) -> None:
        """
        Set the amount of durability to be removed from the tool each time a
        block is broken.

        Arguments
        - damage: the damage to set. Must be 0 or a positive integer
        """
        ...


    def getRules(self) -> list["ToolRule"]:
        """
        Get the list of ToolRule ToolRules that apply to this tool.

        Returns
        - all tool rules. The mutability of the returned list cannot be
        guaranteed, but its contents are mutable and can have their values
        changed
        """
        ...


    def setRules(self, rules: list["ToolRule"]) -> None:
        """
        Set the list of ToolRule ToolRules to apply to this tool. This
        will remove any existing tool rules.

        Arguments
        - rules: the rules to set
        """
        ...


    def addRule(self, block: "Material", speed: "Float", correctForDrops: "Boolean") -> "ToolRule":
        """
        Add a new rule to this tool component, which provides further information
        about a specific block type.

        Arguments
        - block: the block type to which the rule applies
        - speed: the mining speed to use when mining the block, or null to
        use the default mining speed
        - correctForDrops: whether or not this tool, when mining the block,
        is considered the optimal tool for the block and will drop its items when
        broken, or null to use the default tool checking behavior defined by
        Minecraft

        Returns
        - the ToolRule instance that was added to this tool
        """
        ...


    def addRule(self, blocks: Iterable["Material"], speed: "Float", correctForDrops: "Boolean") -> "ToolRule":
        """
        Add a new rule to this tool component, which provides further information
        about a collection of block types.

        Arguments
        - blocks: the block types to which the rule applies
        - speed: the mining speed to use when mining one of the blocks, or
        null to use the default mining speed
        - correctForDrops: whether or not this tool, when mining one of the
        blocks, is considered the optimal tool for the block and will drop its
        items when broken, or null to use the default tool checking behavior
        defined by Minecraft

        Returns
        - the ToolRule instance that was added to this tool
        """
        ...


    def addRule(self, tag: "Tag"["Material"], speed: "Float", correctForDrops: "Boolean") -> "ToolRule":
        """
        Add a new rule to this tool component, which provides further information
        about a collection of block types represented by a block Tag.

        Arguments
        - tag: the block tag containing block types to which the rule
        applies.
        - speed: the mining speed to use when mining one of the blocks, or
        null to use the default mining speed
        - correctForDrops: whether or not this tool, when mining one of the
        blocks, is considered the optimal tool for the block and will drop its
        items when broken, or null to use the default tool checking behavior
        defined by Minecraft

        Returns
        - the ToolRule instance that was added to this tool

        Raises
        - IllegalArgumentException: if the passed `tag` is not a block
        tag
        """
        ...


    def removeRule(self, rule: "ToolRule") -> bool:
        """
        Remove the given ToolRule from this tool.

        Arguments
        - rule: the rule to remove

        Returns
        - True if the rule was removed, False if this component did not
        contain a matching rule
        """
        ...


    class ToolRule(ConfigurationSerializable):
        """
        A rule governing use of this tool and overriding attributes per-block.
        """

        def getBlocks(self) -> Iterable["Material"]:
            """
            Get a collection of the block types to which this tool rule applies.

            Returns
            - the blocks
            """
            ...


        def setBlocks(self, block: "Material") -> None:
            """
            Set the block type to which this rule applies.

            Arguments
            - block: the block type
            """
            ...


        def setBlocks(self, blocks: Iterable["Material"]) -> None:
            """
            Set the block types to which this rule applies.

            Arguments
            - blocks: the block types
            """
            ...


        def setBlocks(self, tag: "Tag"["Material"]) -> None:
            """
            Set the block types (represented as a block Tag) to which
            this rule applies.

            Arguments
            - tag: the block tag

            Raises
            - IllegalArgumentException: if the passed `tag` is not a
            block tag
            """
            ...


        def getSpeed(self) -> "Float":
            """
            Get the mining speed of this rule. If non-null, this speed value is
            used in lieu of the default speed value of the tool. 1.0 is standard
            mining speed.

            Returns
            - the mining speed, or null if the default speed is used
            """
            ...


        def setSpeed(self, speed: "Float") -> None:
            """
            Set the mining speed of this rule. 1.0 is standard mining speed.

            Arguments
            - speed: the mining speed, or null to use the default speed
            """
            ...


        def isCorrectForDrops(self) -> "Boolean":
            """
            Get whether or not this rule is considered the optimal tool for the
            blocks listed by this rule and will drop items. If non-null, this
            value is used in lieu of the default tool checking behavior defined
            by Minecraft.

            Returns
            - True if correct for drops, False otherwise, or null to
            fallback to vanilla tool checking behavior
            """
            ...


        def setCorrectForDrops(self, correct: "Boolean") -> None:
            """
            Set whether or not this rule is considered the optimal tool for the
            blocks listed by this rule and will drop items.

            Arguments
            - correct: whether or not this rule is correct for drops, or null
            to fallback to vanilla tool checking behavior
            """
            ...



"""
Python module generated from Java source file com.google.common.base.Optional

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from com.google.errorprone.annotations import DoNotMock
from java.io import Serializable
from java.util import Iterator
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Optional(Serializable):
    """
    An immutable object that may contain a non-null reference to another object. Each instance of
    this type either contains a non-null reference, or contains nothing (in which case we say that
    the reference is "absent"); it is never said to "contain `null`".
    
    A non-null `Optional<T>` reference can be used as a replacement for a nullable `T`
    reference. It allows you to represent "a `T` that must be present" and a "a `T` that
    might be absent" as two distinct types in your program, which can aid clarity.
    
    Some uses of this class include
    
    
      - As a method return type, as an alternative to returning `null` to indicate that no
          value was available
      - To distinguish between "unknown" (for example, not present in a map) and "known to have no
          value" (present in the map, with value `Optional.absent()`)
      - To wrap nullable references for storage in a collection that does not support `null`
          (though there are <a
          href="https://github.com/google/guava/wiki/LivingWithNullHostileCollections">several other
          approaches to this</a> that should be considered first)
    
    
    A common alternative to using this class is to find or create a suitable <a
    href="http://en.wikipedia.org/wiki/Null_Object_pattern">null object</a> for the type in question.
    
    This class is not intended as a direct analogue of any existing "option" or "maybe" construct
    from other programming environments, though it may bear some similarities.
    
    An instance of this class is serializable if its reference is absent or is a serializable
    object.
    
    **Comparison to `java.util.Optional` (JDK 8 and higher):** A new `Optional`
    class was added for Java 8. The two classes are extremely similar, but incompatible (they cannot
    share a common supertype). *All* known differences are listed either here or with the
    relevant methods below.
    
    
      - This class is serializable; `java.util.Optional` is not.
      - `java.util.Optional` has the additional methods `ifPresent`, `filter`,
          `flatMap`, and `orElseThrow`.
      - `java.util` offers the primitive-specialized versions `OptionalInt`, `OptionalLong` and `OptionalDouble`, the use of which is recommended; Guava does not
          have these.
    
    
    **There are no plans to deprecate this class in the foreseeable future.** However, we do
    gently recommend that you prefer the new, standard Java class whenever possible.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/UsingAndAvoidingNullExplained#optional">using `Optional`</a>.
    
    Type `<T>`: the type of instance that can be contained. `Optional` is naturally covariant on
        this type, so it is safe to cast an `Optional<T>` to `Optional<S>` for any
        supertype `S` of `T`.

    Author(s)
    - Kevin Bourrillion

    Since
    - 10.0
    """

    @staticmethod
    def absent() -> "Optional"["T"]:
        """
        Returns an `Optional` instance with no contained reference.
        
        **Comparison to `java.util.Optional`:** this method is equivalent to Java 8's
        `Optional.empty`.
        """
        ...


    @staticmethod
    def of(reference: "T") -> "Optional"["T"]:
        """
        Returns an `Optional` instance containing the given non-null reference. To have `null` treated as .absent, use .fromNullable instead.
        
        **Comparison to `java.util.Optional`:** no differences.

        Raises
        - NullPointerException: if `reference` is null
        """
        ...


    @staticmethod
    def fromNullable(nullableReference: "T") -> "Optional"["T"]:
        """
        If `nullableReference` is non-null, returns an `Optional` instance containing that
        reference; otherwise returns Optional.absent.
        
        **Comparison to `java.util.Optional`:** this method is equivalent to Java 8's
        `Optional.ofNullable`.
        """
        ...


    @staticmethod
    def fromJavaUtil(javaUtilOptional: "java.util.Optional"["T"]) -> "Optional"["T"]:
        """
        Returns the equivalent `com.google.common.base.Optional` value to the given `java.util.Optional`, or `null` if the argument is null.

        Since
        - 21.0
        """
        ...


    @staticmethod
    def toJavaUtil(googleOptional: "Optional"["T"]) -> "java.util.Optional"["T"]:
        """
        Returns the equivalent `java.util.Optional` value to the given `com.google.common.base.Optional`, or `null` if the argument is null.
        
        If `googleOptional` is known to be non-null, use `googleOptional.toJavaUtil()`
        instead.
        
        Unfortunately, the method reference `Optional::toJavaUtil` will not work, because it
        could refer to either the static or instance version of this method. Write out the lambda
        expression `o -> Optional.toJavaUtil(o)` instead.

        Since
        - 21.0
        """
        ...


    def toJavaUtil(self) -> "java.util.Optional"["T"]:
        """
        Returns the equivalent `java.util.Optional` value to this optional.
        
        Unfortunately, the method reference `Optional::toJavaUtil` will not work, because it
        could refer to either the static or instance version of this method. Write out the lambda
        expression `o -> o.toJavaUtil()` instead.

        Since
        - 21.0
        """
        ...


    def isPresent(self) -> bool:
        """
        Returns `True` if this holder contains a (non-null) instance.
        
        **Comparison to `java.util.Optional`:** no differences.
        """
        ...


    def get(self) -> "T":
        """
        Returns the contained instance, which must be present. If the instance might be absent, use
        .or(Object) or .orNull instead.
        
        **Comparison to `java.util.Optional`:** when the value is absent, this method
        throws IllegalStateException, whereas the `java.util` counterpart throws java.util.NoSuchElementException NoSuchElementException.

        Raises
        - IllegalStateException: if the instance is absent (.isPresent returns `False`); depending on this *specific* exception type (over the more general RuntimeException) is discouraged
        """
        ...


    def or(self, defaultValue: "T") -> "T":
        """
        Returns the contained instance if it is present; `defaultValue` otherwise. If no default
        value should be required because the instance is known to be present, use .get()
        instead. For a default value of `null`, use .orNull.
        
        Note about generics: The signature `public T or(T defaultValue)` is overly
        restrictive. However, the ideal signature, `public <S super T> S or(S)`, is not legal
        Java. As a result, some sensible operations involving subtypes are compile errors:
        
        ````Optional<Integer> optionalInt = getSomeOptionalInt();
        Number value = optionalInt.or(0.5); // error
        
        FluentIterable<? extends Number> numbers = getSomeNumbers();
        Optional<? extends Number> first = numbers.first();
        Number value = first.or(0.5); // error````
        
        As a workaround, it is always safe to cast an `Optional<? extends T>` to `Optional<T>`. Casting either of the above example `Optional` instances to `Optional<Number>` (where `Number` is the desired output type) solves the problem:
        
        ````Optional<Number> optionalInt = (Optional) getSomeOptionalInt();
        Number value = optionalInt.or(0.5); // fine
        
        FluentIterable<? extends Number> numbers = getSomeNumbers();
        Optional<Number> first = (Optional) numbers.first();
        Number value = first.or(0.5); // fine````
        
        **Comparison to `java.util.Optional`:** this method is similar to Java 8's `Optional.orElse`, but will not accept `null` as a `defaultValue` (.orNull
        must be used instead). As a result, the value returned by this method is guaranteed non-null,
        which is not the case for the `java.util` equivalent.
        """
        ...


    def or(self, secondChoice: "Optional"["T"]) -> "Optional"["T"]:
        """
        Returns this `Optional` if it has a value present; `secondChoice` otherwise.
        
        **Comparison to `java.util.Optional`:** this method has no equivalent in Java 8's
        `Optional` class; write `thisOptional.isPresent() ? thisOptional : secondChoice`
        instead.
        """
        ...


    def or(self, supplier: "Supplier"["T"]) -> "T":
        """
        Returns the contained instance if it is present; `supplier.get()` otherwise.
        
        **Comparison to `java.util.Optional`:** this method is similar to Java 8's `Optional.orElseGet`, except when `supplier` returns `null`. In this case this
        method throws an exception, whereas the Java 8+ method returns the `null` to the caller.

        Raises
        - NullPointerException: if this optional's value is absent and the supplier returns `null`
        """
        ...


    def orNull(self) -> "T":
        """
        Returns the contained instance if it is present; `null` otherwise. If the instance is
        known to be present, use .get() instead.
        
        **Comparison to `java.util.Optional`:** this method is equivalent to Java 8's
        `Optional.orElse(null)`.
        """
        ...


    def asSet(self) -> set["T"]:
        """
        Returns an immutable singleton Set whose only element is the contained instance if it
        is present; an empty immutable Set otherwise.
        
        **Comparison to `java.util.Optional`:** this method has no equivalent in Java 8's
        `Optional` class. However, this common usage:
        
        ````for (Foo foo : possibleFoo.asSet()) {
          doSomethingWith(foo);`
        }```
        
        ... can be replaced with:
        
        ````possibleFoo.ifPresent(foo -> doSomethingWith(foo));````
        
        **Java 9 users:** some use cases can be written with calls to `optional.stream()`.

        Since
        - 11.0
        """
        ...


    def transform(self, function: "Function"["T", "V"]) -> "Optional"["V"]:
        """
        If the instance is present, it is transformed with the given Function; otherwise,
        Optional.absent is returned.
        
        **Comparison to `java.util.Optional`:** this method is similar to Java 8's `Optional.map`, except when `function` returns `null`. In this case this method
        throws an exception, whereas the Java 8+ method returns `Optional.absent()`.

        Raises
        - NullPointerException: if the function returns `null`

        Since
        - 12.0
        """
        ...


    def equals(self, object: "Object") -> bool:
        """
        Returns `True` if `object` is an `Optional` instance, and either the
        contained references are Object.equals equal to each other or both are absent.
        Note that `Optional` instances of differing parameterized types can be equal.
        
        **Comparison to `java.util.Optional`:** no differences.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hash code for this instance.
        
        **Comparison to `java.util.Optional`:** this class leaves the specific choice of
        hash code unspecified, unlike the Java 8+ equivalent.
        """
        ...


    def toString(self) -> str:
        """
        Returns a string representation for this instance.
        
        **Comparison to `java.util.Optional`:** this class leaves the specific string
        representation unspecified, unlike the Java 8+ equivalent.
        """
        ...


    @staticmethod
    def presentInstances(optionals: Iterable["Optional"["T"]]) -> Iterable["T"]:
        """
        Returns the value of each present instance from the supplied `optionals`, in order,
        skipping over occurrences of Optional.absent. Iterators are unmodifiable and are
        evaluated lazily.
        
        **Comparison to `java.util.Optional`:** this method has no equivalent in Java 8's
        `Optional` class; use `optionals.stream().filter(Optional::isPresent).map(Optional::get)` instead.
        
        **Java 9 users:** use `optionals.stream().flatMap(Optional::stream)` instead.

        Since
        - 11.0 (generics widened in 13.0)
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.CrafterCraftEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from org.bukkit.inventory import CraftingRecipe
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class CrafterCraftEvent(BlockEvent, Cancellable):
    """
    Event called when a Crafter is about to craft an item.
    """

    def __init__(self, theBlock: "Block", recipe: "CraftingRecipe", result: "ItemStack"):
        ...


    def getResult(self) -> "ItemStack":
        """
        Gets the result for the craft.

        Returns
        - the result for the craft
        """
        ...


    def setResult(self, result: "ItemStack") -> None:
        """
        Sets the result of the craft.

        Arguments
        - result: the result of the craft
        """
        ...


    def getRecipe(self) -> "CraftingRecipe":
        """
        Gets the recipe that was used to craft this item.

        Returns
        - the recipe that was used to craft this item
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.SetMultimap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class SetMultimap(Multimap):
    """
    A `Multimap` that cannot hold duplicate key-value pairs. Adding a key-value pair that's
    already in the multimap has no effect. See the Multimap documentation for information
    common to all multimaps.
    
    The .get, .removeAll, and .replaceValues methods each return a Set of values, while .entries returns a `Set` of map entries. Though the method
    signature doesn't say so explicitly, the map returned by .asMap has `Set` values.
    
    If the values corresponding to a single key should be ordered according to a java.util.Comparator (or the natural order), see the SortedSetMultimap subinterface.
    
    Since the value collections are sets, the behavior of a `SetMultimap` is not specified
    if key *or value* objects already present in the multimap change in a manner that affects
    `equals` comparisons. Use caution if mutable objects are used as keys or values in a `SetMultimap`.
    
    **Warning:** Do not modify either a key *or a value* of a `SetMultimap` in a way
    that affects its Object.equals behavior. Undefined behavior and bugs will result.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#multimap">`Multimap`</a>.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    def get(self, key: "K") -> set["V"]:
        """
        
        
        Because a `SetMultimap` has unique values for a given key, this method returns a
        Set, instead of the java.util.Collection specified in the Multimap
        interface.
        """
        ...


    def removeAll(self, key: "Object") -> set["V"]:
        """
        
        
        Because a `SetMultimap` has unique values for a given key, this method returns a
        Set, instead of the java.util.Collection specified in the Multimap
        interface.
        """
        ...


    def replaceValues(self, key: "K", values: Iterable["V"]) -> set["V"]:
        """
        
        
        Because a `SetMultimap` has unique values for a given key, this method returns a
        Set, instead of the java.util.Collection specified in the Multimap
        interface.
        
        Any duplicates in `values` will be stored in the multimap once.
        """
        ...


    def entries(self) -> set["Entry"["K", "V"]]:
        """
        
        
        Because a `SetMultimap` has unique values for a given key, this method returns a
        Set, instead of the java.util.Collection specified in the Multimap
        interface.
        """
        ...


    def asMap(self) -> dict["K", Iterable["V"]]:
        """
        
        
        **Note:** The returned map's values are guaranteed to be of type Set. To obtain
        this map with the more specific generic type `Map<K, Set<V>>`, call Multimaps.asMap(SetMultimap) instead.
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Compares the specified object to this multimap for equality.
        
        Two `SetMultimap` instances are equal if, for each key, they contain the same values.
        Equality does not depend on the ordering of keys or values.
        
        An empty `SetMultimap` is equal to any other empty `Multimap`, including an
        empty `ListMultimap`.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ForwardingTable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ForwardingTable(ForwardingObject, Table):
    """
    A table which forwards all its method calls to another table. Subclasses should override one or
    more methods to modify the behavior of the backing map as desired per the <a
    href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.

    Author(s)
    - Gregory Kick

    Since
    - 7.0
    """

    def cellSet(self) -> set["Cell"["R", "C", "V"]]:
        ...


    def clear(self) -> None:
        ...


    def column(self, columnKey: "C") -> dict["R", "V"]:
        ...


    def columnKeySet(self) -> set["C"]:
        ...


    def columnMap(self) -> dict["C", dict["R", "V"]]:
        ...


    def contains(self, rowKey: "Object", columnKey: "Object") -> bool:
        ...


    def containsColumn(self, columnKey: "Object") -> bool:
        ...


    def containsRow(self, rowKey: "Object") -> bool:
        ...


    def containsValue(self, value: "Object") -> bool:
        ...


    def get(self, rowKey: "Object", columnKey: "Object") -> "V":
        ...


    def isEmpty(self) -> bool:
        ...


    def put(self, rowKey: "R", columnKey: "C", value: "V") -> "V":
        ...


    def putAll(self, table: "Table"["R", "C", "V"]) -> None:
        ...


    def remove(self, rowKey: "Object", columnKey: "Object") -> "V":
        ...


    def row(self, rowKey: "R") -> dict["C", "V"]:
        ...


    def rowKeySet(self) -> set["R"]:
        ...


    def rowMap(self) -> dict["R", dict["C", "V"]]:
        ...


    def size(self) -> int:
        ...


    def values(self) -> Iterable["V"]:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...



"""
Python module generated from Java source file org.bukkit.Particle

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from enum import Enum
from org.bukkit import *
from org.bukkit.block.data import BlockData
from org.bukkit.inventory import ItemStack
from org.bukkit.registry import RegistryAware
from typing import Any, Callable, Iterable, Tuple


class Particle(Enum):

    POOF = ("poof")
    EXPLOSION = ("explosion")
    EXPLOSION_EMITTER = ("explosion_emitter")
    FIREWORK = ("firework")
    BUBBLE = ("bubble")
    SPLASH = ("splash")
    FISHING = ("fishing")
    UNDERWATER = ("underwater")
    CRIT = ("crit")
    ENCHANTED_HIT = ("enchanted_hit")
    SMOKE = ("smoke")
    LARGE_SMOKE = ("large_smoke")
    EFFECT = ("effect")
    INSTANT_EFFECT = ("instant_effect")
    ENTITY_EFFECT = ("entity_effect", Color)
    """
    Uses Color as DataType
    """
    WITCH = ("witch")
    DRIPPING_WATER = ("dripping_water")
    DRIPPING_LAVA = ("dripping_lava")
    ANGRY_VILLAGER = ("angry_villager")
    HAPPY_VILLAGER = ("happy_villager")
    MYCELIUM = ("mycelium")
    NOTE = ("note")
    PORTAL = ("portal")
    ENCHANT = ("enchant")
    FLAME = ("flame")
    LAVA = ("lava")
    CLOUD = ("cloud")
    DUST = ("dust", DustOptions)
    """
    Uses Particle.DustOptions as DataType
    """
    ITEM_SNOWBALL = ("item_snowball")
    ITEM_SLIME = ("item_slime")
    HEART = ("heart")
    ITEM = ("item", ItemStack)
    """
    Uses ItemStack as DataType
    """
    BLOCK = ("block", BlockData)
    """
    Uses BlockData as DataType
    """
    RAIN = ("rain")
    ELDER_GUARDIAN = ("elder_guardian")
    DRAGON_BREATH = ("dragon_breath")
    END_ROD = ("end_rod")
    DAMAGE_INDICATOR = ("damage_indicator")
    SWEEP_ATTACK = ("sweep_attack")
    FALLING_DUST = ("falling_dust", BlockData)
    """
    Uses BlockData as DataType
    """
    TOTEM_OF_UNDYING = ("totem_of_undying")
    SPIT = ("spit")
    SQUID_INK = ("squid_ink")
    BUBBLE_POP = ("bubble_pop")
    CURRENT_DOWN = ("current_down")
    BUBBLE_COLUMN_UP = ("bubble_column_up")
    NAUTILUS = ("nautilus")
    DOLPHIN = ("dolphin")
    SNEEZE = ("sneeze")
    CAMPFIRE_COSY_SMOKE = ("campfire_cosy_smoke")
    CAMPFIRE_SIGNAL_SMOKE = ("campfire_signal_smoke")
    COMPOSTER = ("composter")
    FLASH = ("flash")
    FALLING_LAVA = ("falling_lava")
    LANDING_LAVA = ("landing_lava")
    FALLING_WATER = ("falling_water")
    DRIPPING_HONEY = ("dripping_honey")
    FALLING_HONEY = ("falling_honey")
    LANDING_HONEY = ("landing_honey")
    FALLING_NECTAR = ("falling_nectar")
    SOUL_FIRE_FLAME = ("soul_fire_flame")
    ASH = ("ash")
    CRIMSON_SPORE = ("crimson_spore")
    WARPED_SPORE = ("warped_spore")
    SOUL = ("soul")
    DRIPPING_OBSIDIAN_TEAR = ("dripping_obsidian_tear")
    FALLING_OBSIDIAN_TEAR = ("falling_obsidian_tear")
    LANDING_OBSIDIAN_TEAR = ("landing_obsidian_tear")
    REVERSE_PORTAL = ("reverse_portal")
    WHITE_ASH = ("white_ash")
    DUST_COLOR_TRANSITION = ("dust_color_transition", DustTransition)
    """
    Uses DustTransition as DataType
    """
    VIBRATION = ("vibration", Vibration)
    """
    Uses Vibration as DataType
    """
    FALLING_SPORE_BLOSSOM = ("falling_spore_blossom")
    SPORE_BLOSSOM_AIR = ("spore_blossom_air")
    SMALL_FLAME = ("small_flame")
    SNOWFLAKE = ("snowflake")
    DRIPPING_DRIPSTONE_LAVA = ("dripping_dripstone_lava")
    FALLING_DRIPSTONE_LAVA = ("falling_dripstone_lava")
    DRIPPING_DRIPSTONE_WATER = ("dripping_dripstone_water")
    FALLING_DRIPSTONE_WATER = ("falling_dripstone_water")
    GLOW_SQUID_INK = ("glow_squid_ink")
    GLOW = ("glow")
    WAX_ON = ("wax_on")
    WAX_OFF = ("wax_off")
    ELECTRIC_SPARK = ("electric_spark")
    SCRAPE = ("scrape")
    SONIC_BOOM = ("sonic_boom")
    SCULK_SOUL = ("sculk_soul")
    SCULK_CHARGE = ("sculk_charge", Float)
    """
    Use Float as DataType
    """
    SCULK_CHARGE_POP = ("sculk_charge_pop")
    SHRIEK = ("shriek", Integer)
    """
    Use Integer as DataType
    """
    CHERRY_LEAVES = ("cherry_leaves")
    PALE_OAK_LEAVES = ("pale_oak_leaves")
    EGG_CRACK = ("egg_crack")
    DUST_PLUME = ("dust_plume")
    WHITE_SMOKE = ("white_smoke")
    GUST = ("gust")
    SMALL_GUST = ("small_gust")
    GUST_EMITTER_LARGE = ("gust_emitter_large")
    GUST_EMITTER_SMALL = ("gust_emitter_small")
    TRIAL_SPAWNER_DETECTION = ("trial_spawner_detection")
    TRIAL_SPAWNER_DETECTION_OMINOUS = ("trial_spawner_detection_ominous")
    VAULT_CONNECTION = ("vault_connection")
    INFESTED = ("infested")
    ITEM_COBWEB = ("item_cobweb")
    DUST_PILLAR = ("dust_pillar", BlockData)
    """
    Uses BlockData as DataType
    """
    BLOCK_CRUMBLE = ("block_crumble", BlockData)
    """
    Uses BlockData as DataType
    """
    TRAIL = ("trail", Trail)
    """
    Uses Trail as DataType
    """
    OMINOUS_SPAWNING = ("ominous_spawning")
    RAID_OMEN = ("raid_omen")
    TRIAL_OMEN = ("trial_omen")
    BLOCK_MARKER = ("block_marker", BlockData)
    """
    Uses BlockData as DataType
    """


    def getDataType(self) -> type[Any]:
        """
        Returns the required data type for the particle

        Returns
        - the required data type
        """
        ...


    def getKeyOrThrow(self) -> "NamespacedKey":
        ...


    def getKeyOrNull(self) -> "NamespacedKey":
        ...


    def isRegistered(self) -> bool:
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...


    class DustOptions:
        """
        Options which can be applied to redstone dust particles - a particle
        color and size.
        """

        def __init__(self, color: "Color", size: float):
            ...


        def getColor(self) -> "Color":
            """
            The color of the particles to be displayed.

            Returns
            - particle color
            """
            ...


        def getSize(self) -> float:
            """
            Relative size of the particle.

            Returns
            - relative particle size
            """
            ...


    class DustTransition(DustOptions):
        """
        Options which can be applied to a color transitioning dust particles.
        """

        def __init__(self, fromColor: "Color", toColor: "Color", size: float):
            ...


        def getToColor(self) -> "Color":
            """
            The final of the particles to be displayed.

            Returns
            - final particle color
            """
            ...


    class Trail:
        """
        Options which can be applied to trail particles - a location, color and duration.
        """

        def __init__(self, target: "Location", color: "Color", duration: int):
            ...


        def getTarget(self) -> "Location":
            """
            The target of the particles to be displayed.

            Returns
            - particle target
            """
            ...


        def getColor(self) -> "Color":
            """
            The color of the particles to be displayed.

            Returns
            - particle color
            """
            ...


        def getDuration(self) -> int:
            """
            The duration of the trail to be displayed.

            Returns
            - trail duration
            """
            ...



"""
Python module generated from Java source file org.bukkit.permissions.PermissibleBase

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Locale
from org.bukkit import Bukkit
from org.bukkit.permissions import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class PermissibleBase(Permissible):
    """
    Base Permissible for use in any Permissible object via proxy or extension
    """

    def __init__(self, opable: "ServerOperator"):
        ...


    def isOp(self) -> bool:
        ...


    def setOp(self, value: bool) -> None:
        ...


    def isPermissionSet(self, name: str) -> bool:
        ...


    def isPermissionSet(self, perm: "Permission") -> bool:
        ...


    def hasPermission(self, inName: str) -> bool:
        ...


    def hasPermission(self, perm: "Permission") -> bool:
        ...


    def addAttachment(self, plugin: "Plugin", name: str, value: bool) -> "PermissionAttachment":
        ...


    def addAttachment(self, plugin: "Plugin") -> "PermissionAttachment":
        ...


    def removeAttachment(self, attachment: "PermissionAttachment") -> None:
        ...


    def recalculatePermissions(self) -> None:
        ...


    def clearPermissions(self) -> None:
        ...


    def addAttachment(self, plugin: "Plugin", name: str, value: bool, ticks: int) -> "PermissionAttachment":
        ...


    def addAttachment(self, plugin: "Plugin", ticks: int) -> "PermissionAttachment":
        ...


    def getEffectivePermissions(self) -> set["PermissionAttachmentInfo"]:
        ...



"""
Python module generated from Java source file com.google.common.collect.ConsumingQueueIterator

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from java.util import Queue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ConsumingQueueIterator(AbstractIterator):
    """
    An Iterator implementation which draws elements from a queue, removing them from the queue as it
    iterates. This class is not thread safe.
    """





"""
Python module generated from Java source file org.bukkit.map.MinecraftFont

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.map import *
from typing import Any, Callable, Iterable, Tuple


class MinecraftFont(MapFont):
    """
    Represents the built-in Minecraft font.
    """

    Font = MinecraftFont(false)
    """
    A static non-malleable MinecraftFont.
    """


    def __init__(self):
        """
        Initialize a new MinecraftFont.
        """
        ...



"""
Python module generated from Java source file java.nio.file.attribute.FileTime

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.file.attribute import *
from java.time import Instant
from java.time import LocalDateTime
from java.time import ZoneOffset
from java.util import Objects
from java.util.concurrent import TimeUnit
from typing import Any, Callable, Iterable, Tuple


class FileTime(Comparable):

    @staticmethod
    def from(value: int, unit: "TimeUnit") -> "FileTime":
        """
        Returns a `FileTime` representing a value at the given unit of
        granularity.

        Arguments
        - value: the value since the epoch (1970-01-01T00:00:00Z); can be
                 negative
        - unit: the unit of granularity to interpret the value

        Returns
        - a `FileTime` representing the given value
        """
        ...


    @staticmethod
    def fromMillis(value: int) -> "FileTime":
        """
        Returns a `FileTime` representing the given value in milliseconds.

        Arguments
        - value: the value, in milliseconds, since the epoch
                 (1970-01-01T00:00:00Z); can be negative

        Returns
        - a `FileTime` representing the given value
        """
        ...


    @staticmethod
    def from(instant: "Instant") -> "FileTime":
        """
        Returns a `FileTime` representing the same point of time value
        on the time-line as the provided `Instant` object.

        Arguments
        - instant: the instant to convert

        Returns
        - a `FileTime` representing the same point on the time-line
                 as the provided instant

        Since
        - 1.8
        """
        ...


    def to(self, unit: "TimeUnit") -> int:
        """
        Returns the value at the given unit of granularity.
        
         Conversion from a coarser granularity that would numerically overflow
        saturate to `Long.MIN_VALUE` if negative or `Long.MAX_VALUE`
        if positive.

        Arguments
        - unit: the unit of granularity for the return value

        Returns
        - value in the given unit of granularity, since the epoch
                 since the epoch (1970-01-01T00:00:00Z); can be negative
        """
        ...


    def toMillis(self) -> int:
        """
        Returns the value in milliseconds.
        
         Conversion from a coarser granularity that would numerically overflow
        saturate to `Long.MIN_VALUE` if negative or `Long.MAX_VALUE`
        if positive.

        Returns
        - the value in milliseconds, since the epoch (1970-01-01T00:00:00Z)
        """
        ...


    def toInstant(self) -> "Instant":
        """
        Converts this `FileTime` object to an `Instant`.
        
         The conversion creates an `Instant` that represents the
        same point on the time-line as this `FileTime`.
        
         `FileTime` can store points on the time-line further in the
        future and further in the past than `Instant`. Conversion
        from such further time points saturates to Instant.MIN if
        earlier than `Instant.MIN` or Instant.MAX if later
        than `Instant.MAX`.

        Returns
        - an instant representing the same point on the time-line as
                 this `FileTime` object

        Since
        - 1.8
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Tests this `FileTime` for equality with the given object.
        
         The result is `True` if and only if the argument is not `null` and is a `FileTime` that represents the same time. This
        method satisfies the general contract of the `Object.equals` method.

        Arguments
        - obj: the object to compare with

        Returns
        - `True` if, and only if, the given object is a `FileTime` that represents the same time
        """
        ...


    def hashCode(self) -> int:
        """
        Computes a hash code for this file time.
        
         The hash code is based upon the value represented, and satisfies the
        general contract of the Object.hashCode method.

        Returns
        - the hash-code value
        """
        ...


    def compareTo(self, other: "FileTime") -> int:
        """
        Compares the value of two `FileTime` objects for order.

        Arguments
        - other: the other `FileTime` to be compared

        Returns
        - `0` if this `FileTime` is equal to `other`, a
                 value less than 0 if this `FileTime` represents a time
                 that is before `other`, and a value greater than 0 if this
                 `FileTime` represents a time that is after `other`
        """
        ...


    def toString(self) -> str:
        """
        Returns the string representation of this `FileTime`. The string
        is returned in the <a
        href="http://www.w3.org/TR/NOTE-datetime">ISO&nbsp;8601</a> format:
        ```
            YYYY-MM-DDThh:mm:ss[.s+]Z
        ```
        where "`[.s+]`" represents a dot followed by one of more digits
        for the decimal fraction of a second. It is only present when the decimal
        fraction of a second is not zero. For example, `FileTime.fromMillis(1234567890000L).toString()` yields `"2009-02-13T23:31:30Z"`, and `FileTime.fromMillis(1234567890123L).toString()`
        yields `"2009-02-13T23:31:30.123Z"`.
        
         A `FileTime` is primarily intended to represent the value of a
        file's time stamp. Where used to represent *extreme values*, where
        the year is less than "`0001`" or greater than "`9999`" then
        this method deviates from ISO 8601 in the same manner as the
        <a href="http://www.w3.org/TR/xmlschema-2/#deviantformats">XML Schema
        language</a>. That is, the year may be expanded to more than four digits
        and may be negative-signed. If more than four digits then leading zeros
        are not present. The year before "`0001`" is "`-0001`".

        Returns
        - the string representation of this file time
        """
        ...
